import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as tck
import scipy.stats

import helpers
import EPOS.multi
from EPOS.population import periodradius as draw_PR
from EPOS.population import periodratio as draw_dP

# backwards compatible colors 
import matplotlib
if matplotlib.__version__[0] != 2: 
	helpers.default_pyplot2_colors(matplotlib.colors)

def periodradius(epos, Nth=False, MC=True):

	f, (ax, axR, axP)= helpers.make_panels(plt)
	
	if MC:
		sim=epos.synthetic_survey
		ID= sim['ID']
		P= sim['P']
		Y= sim['Y']
		outdir='output'
		title= 'Simulated Detections'
	else:
		ID= epos.obs_starID
		P= epos.obs_xvar
		Y= epos.obs_yvar
		outdir='survey'
		title= r'Planet Candidates (score$\geq$0.9)'
	
	''' plot R(P), main panel'''
	ax.set_title(title)
	helpers.set_axes(ax, epos, Trim=True)
		
	''' Period side panel '''
	helpers.set_axis_distance(axP, epos, Trim=True)
	#axP.set_yscale('log')
	#axP.set_ylim([2e-3,5])	
	#axP.set_yticks([0.01,0.1,1])
	#axP.set_yticklabels(['1%','10%','100%'])
	axP.yaxis.tick_right()
	axP.yaxis.set_ticks_position('both')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')
	
	#axP.hist(sim['P'], bins=epos.MC_xvar, color='0.7')

	''' Radius side panel'''
	helpers.set_axis_size(axR, epos, Trim=True, In= epos.MassRadius)

	#axR.set_xscale('log')
	#axR.set_xlim([2e-3,5])
	#axR.set_xticks([1,10,100,1000])
	#axR.set_xticklabels(['1','10','100','1000'], rotation=70)
	for tick in axR.get_xticklabels():
		tick.set_rotation(70)
	#axR.tick_params(axis='x', which='minor',top='off',bottom='off')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')

	''' which multiplanets to color '''
	# 	single, multi= EPOS.multi.indices(sim['ID'])
	# 	for k, (label, subset) in enumerate(zip(['single','multi'],[single, multi])):
	# 		ax.plot(sim['P'][subset], sim['Y'][subset], ls='', marker='.', mew=0, ms=5.0, \
	# 			label=label)

	if Nth:
		single, multi, ksys, multis= EPOS.multi.nth_planet(ID, P)
		suffix= '.nth'
		label_single='single'
	else:
		single, multi, ksys, multis= EPOS.multi.indices(ID)
		suffix= ''
		label_single='1'
	
	ax.plot(P[single], Y[single], ls='', marker='.', \
			color='0.7', label=label_single)

	''' Multiplanets with colors'''
	Stacked=True
	plist=[]; ylist=[]; colors=[]
	CDF= False
	#stacked histogram
			
	for k, subset in zip(ksys, multis):
		ht, = ax.plot(P[subset], Y[subset], ls='', marker='.', \
			label=k)
		
		if not CDF:
			if Stacked:
				plist.insert(0,P[subset])
				ylist.insert(0,Y[subset])
				colors.insert(0, ht.get_color())	
			# pdf
			else:
				axP.hist(P[subset], bins=epos.MC_xvar, 
					color=ht.get_color(), histtype='step')
				if k==ksys[-1]:
					axP.hist(P[single],bins=epos.MC_xvar,color='0.7',histtype='step')	
		else:
			# cumulative
			Plist= np.sort(P[subset])
			axP.step(Plist,np.arange(Plist.size,dtype=float)/Plist.size )

	if Stacked:
		plist.append(P[single])
		ylist.append(Y[single])
		colors.append('0.7')
		axP.hist(plist, bins=epos.MC_xvar, 
			color=colors, histtype='barstacked')
		axR.hist(ylist, bins=epos.MC_yvar, orientation='horizontal',
			color=colors, histtype='barstacked')
		
	else:
		#axR.hist(Y,orientation='horizontal', bins=epos.MC_yvar, color='0.7')
		axR.hist(Y,orientation='horizontal', bins=epos.MC_yvar, color='k',histtype='step')
		axR.hist(Y[single],orientation='horizontal', bins=epos.MC_yvar, color='0.7')

	
	#ax.legend(loc='lower left', shadow=False, prop={'size':14}, numpoints=1)
	ax.legend(bbox_to_anchor=(1.0, 1.0), markerscale=2, frameon=True, 
		borderpad=0.2, handlelength=1, handletextpad=0.2)

	helpers.save(plt, '{}{}/PR.multi{}'.format(epos.plotdir, outdir, suffix))

def multiplicity(epos, MC=False, Planets=False, MCMC=False, color='C1'):
	# plot multiplicity
	f, ax = plt.subplots()
	ax.set_title('Multi-Planet Frequency')
	ax.set_xlabel('Planets per System')
	ax.set_ylabel('Planet Counts' if Planets else 'System Counts')

	if hasattr(epos, 'pfm'):
		ax.set_xlim(0.5, 10.5)
	else:
		ax.set_xlim(0.5, 7.5)

	if not Planets:
		ax.set_ylim(0.5, 1e4) # 7.	
		ax.set_yscale('log')
		#ax.get_yaxis().set_major_formatter(tck.ScalarFormatter())

	key = 'pl cnt' if Planets else 'count'

	# MC data
	if MC or MCMC:
		ss=epos.synthetic_survey
		if Planets:
			ax.bar(ss['multi']['bin'], ss['multi'][key], 
				color=color,label='Simulated', width=1)

			f_iso= epos.fitpars.get('f_iso')
			if f_iso > 0:
				fsingle= np.sum(ss['multi']['count'])*f_iso
				label= 'Singles ({:2.0%})'.format(f_iso)
				ax.bar(1, fsingle, bottom= ss['multi'][key][0]-fsingle, 
					color='',label=label, width=1, hatch='xx') #, ec='k')

			if hasattr(epos, 'pfm'):
				ax.plot(epos.pfm['k'], epos.pfm['Nk']*epos.pfm['k'], 
					color='C7',label='Input', drawstyle='steps-mid', ls=':')

		elif MCMC:
			for ss in epos.ss_sample:
				ax.hlines(ss['multi'][key], ss['multi']['bin']-0.5,ss['multi']['bin']+0.5,
					color='b', alpha=0.1)
				
		else:
			#ax.step(ss['multi']['bin'], ss['multi'][key], color='C0',label=epos.name, where='mid')
			ax.plot(ss['multi']['bin'], ss['multi'][key], 
				ls='', marker='+', ms=10, color=color,label=epos.name)

			if hasattr(epos, 'pfm'):
				ax.plot(epos.pfm['k'], epos.pfm['Nk'], 
					ls=':', marker='x', ms=8, color='C7',label='Input')
		
		if hasattr(epos, 'ss_extra'):
			# advance color cycle
			# ax._get_lines.get_next_color()
			ax.plot([], [])
			ax.plot([], [])

			# plot extra epos runs
			for ss in epos.ss_extra:
				ax.plot(ss['multi']['bin'], ss['multi'][key], 
					ls='', marker='+', ms=10, label=ss['name'])
	
		# observations in same region 
		ax.plot(epos.obs_zoom['multi']['bin'], epos.obs_zoom['multi'][key], 
			drawstyle='steps-mid', ls='--', color='C3', label='Kepler')

	else:
		# observations
		ax.plot(epos.multi['bin'], epos.multi[key], drawstyle='steps-mid', 
			ls='--', marker='', color='gray', label='Kepler all')


	ax.legend(loc='upper right', shadow=False, prop={'size':14}, numpoints=1)
	
	prefix= 'output' if MC else 'survey'
	if MCMC: prefix= 'mcmc'
	suffix='.planets' if Planets else ''
	
	helpers.save(plt, '{}{}/multiplicity{}'.format(epos.plotdir,prefix,suffix))

def multiplicity_cdf(epos, MC=False, color='C1'):
	# plot multiplicity cdf
	f, ax = plt.subplots()
	ax.set_title('planet multiplicity')
	ax.set_xlabel('planets per system')
	ax.set_ylabel('cumulative number of planets')

	ax.set_xlim(0, 9)
	ax.set_ylim(-0.01,1.05)
	#ax.set_yscale('log')

	# MC data
	if MC:
		ss=epos.synthetic_survey
		bincounts= np.sort(ss['multi']['cdf'])
		cdf= np.arange(bincounts.size, dtype=float)/bincounts.size
		ax.plot(bincounts, cdf, 
			drawstyle='steps-mid', 
			ls='-', marker='', mew=2, ms=10, color=color,label=epos.name)
	
		# observations in same region 
		bincounts= np.sort(epos.obs_zoom['multi']['cdf'])
		cdf= np.arange(bincounts.size, dtype=float)/bincounts.size
		ax.plot(bincounts, cdf,  
			drawstyle='steps-mid', 
			ls='--', marker='', color='C3', label='Kepler')

	# observations
	#ax.plot(epos.multi['bin'], epos.multi['count'], drawstyle='steps-mid', 
	#	ls='--', marker='', color='gray', label='Kepler all')
		
	ax.legend(loc='lower right', shadow=False, prop={'size':14}, numpoints=1)
	
	prefix= 'output' if MC else 'survey'
	
	helpers.save(plt, '{}{}/cdf'.format(epos.plotdir,prefix))
	
def periodratio(epos, MC=False, N=False, Input=False, MCMC=False, color='C1'):
	# plot multiplicity
	f, ax = plt.subplots()
	ax.set_title('Period Ratio of Adjacent Planets')
	ax.set_xlabel(r'$\mathcal{P}$ = Period Outer/Inner')
	ax.set_ylabel('Planet Counts')

	ax.set_xlim(1, 10)
	ax.set_xscale('log')
	for s in [ax.set_xticks,ax.set_xticklabels]: s([1,2,3,4,5,7,10])
	ax.set_xticks([], minor=True) # minor ticks generate labels

	bins=np.logspace(0,1, 15) # 1,10
	
	# MC data
	if MC or MCMC:
		ss=epos.synthetic_survey
		if MCMC:
			for ss in epos.ss_sample:
				# bar?
				dP= ss['multi']['Pratio'][ss['multi']['Pratio']>1]
				ax.hist(dP, bins=bins, histtype='step', color='b', alpha=0.1)
				#ax.hist(ss['multi']['Pratio'], bins=bins, color='b', alpha=1./len(epos.ss_sample))
		else:
			dP= ss['multi']['Pratio'][ss['multi']['Pratio']>1]
			ax.hist(dP, bins=bins, 
					color=color, histtype='stepfilled', label='Simulated')
	
			if hasattr(epos, 'pfm'):
				dP_mod= epos.pfm['dP'][epos.pfm['dP']>1.]
				scale= 1.*dP.size/dP_mod.size
				ax.hist(dP_mod, bins=bins, weights=np.full_like(dP_mod, scale),
					color='C7',label='Input/{:.2f}'.format(scale), histtype='step', ls=':')

		if N:
			# planets inbetween?
			#ax.hist(ss['multi']['dPN'][0], \
			#	bins=bins, ec='k', histtype='step', label='Adjacent planet')
			ax.hist(np.concatenate(ss['multi']['dPN'][1:]), 
				bins=bins, hatch='xx',
				histtype='stepfilled', label='Planet Inbetween') # ec, color
		else:
			#ax.axvline(np.median(ss['multi']['Pratio']), color='C0', ls='--')
			pass
		
		if hasattr(epos, 'ss_extra'):
			# advance color cycle
			# ax._get_lines.get_next_color()
			ax.plot([], [])
			ax.plot([], [])
			ax.plot([], [])
			
			# plot extra epos runs
			for ss in epos.ss_extra:
				ax.hist(ss['multi']['Pratio'], bins=bins, 
					histtype='step', label=ss['name'])

		# Observed zoom
		dP_obs= epos.obs_zoom['multi']['Pratio'][epos.obs_zoom['multi']['Pratio']>1.]
		ax.hist(dP_obs, bins=bins, color='C3', histtype='step', ls='--', label='Kepler')


	else:
		# observed all
		ax.hist(epos.multi['Pratio'], bins=bins, color='0.7', label='Kepler all')
		#ax.axvline(np.median(epos.multi['Pratio']), color='0.7', ls='--')
	
	''' input distribution '''	
	if Input and epos.Parametric:
		#Pgrid=bins
		Pgrid= np.logspace(0,1)
		pdf, _= draw_dP(epos, Pgrid=Pgrid)
		pdf*= 0.95* ax.get_ylim()[1] / max(pdf)	
		ax.plot(Pgrid, pdf, marker='', ls='-', color='r',label='Intrinsic')

	elif epos.Zoom:
		if not MCMC:
			ax.axvline(np.median(epos.obs_zoom['multi']['Pratio']), color='C1', ls='--')
	
	prefix= 'output' if MC else 'survey'
	suffix= '.index' if N else ''
	if Input: suffix+= '.input' 
	if MCMC: prefix= 'mcmc'

	if MC: ax.legend(loc='upper right', shadow=False, prop={'size':14}, numpoints=1)
		
	helpers.save(plt, '{}{}/periodratio{}'.format(epos.plotdir, prefix, suffix))	

def periodratio_cdf(epos, Input=True, MC=False, color='C1'):

	# plot multiplicity CDF
	f, ax = plt.subplots()
	ax.set_title('Period Ratio Adjacent Planets')
	#ax.set_xlabel('period outer/inner')
	ax.set_xlabel('$\mathcal{P}$ = Period Outer/Inner')
	ax.set_ylabel('CDF')

	ax.set_xlim(1, 10)
	ax.set_xscale('log')
	for s in [ax.set_xticks,ax.set_xticklabels]: s([1,2,3,4,5,7,10])
	ax.set_xticks([], minor=True) # minor ticks generate labels
	
	# MC data
	if MC:
		ss=epos.synthetic_survey
		dP= ss['multi']['Pratio'][ss['multi']['Pratio']>1.]
		Psort= np.sort(dP)
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf, color=color, label=epos.name)
	
		# Observed zoom
		dP_obs= epos.obs_zoom['multi']['Pratio'][epos.obs_zoom['multi']['Pratio']>1.]
		Psort= np.sort(dP_obs)
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf, color='C3', label='Kepler', ls='--')

		if hasattr(epos, 'pfm'):
			dP= epos.pfm['dP'][epos.pfm['dP']>1.]
			Psort= np.sort(dP)
			cdf= np.arange(Psort.size, dtype=float)/Psort.size
			ax.plot(Psort, cdf, color=color, ls=':', label='Input')			

	else:
		for resonance in [2./1., 3./2.]: ax.axvline(resonance, ls=':', color='g')

		# all observed
		dP= epos.multi['Pratio'][epos.multi['Pratio']>1]
		Psort= np.sort(dP)
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf ,color='C7', ls=':', label='Kepler all')	
	
	# 	''' input distribution '''	
	# 	if Input and epos.Parametric:
	# 		Pgrid= np.logspace(0,1)
	# 		_, cdf= draw_dP(epos, Pgrid=Pgrid)
	# 		ax.plot(Pgrid, cdf, marker='', ls='-', color='r',label='Intrinsic')
	# 	
	# 	# HZ
	# 	ax.axvline(2.6, ls=':',label='Hab zone width')
	# 	ax.axvline(2.6**0.5, ls=':',label='Hab zone 2 planets')
	
	prefix= 'output' if MC else 'survey'
	
	if MC: ax.legend(loc='lower right', shadow=False, prop={'size':14}, numpoints=1)
		
	helpers.save(plt, '{}{}/periodratio.cdf'.format(epos.plotdir,prefix))	

''' these are practically identical to periodratio -> merge?'''

def periodinner(epos, MC=False, N=False, Input=False, MCMC=False, color='C1'):
	# plot multiplicity
	f, ax = plt.subplots()
	ax.set_title('Period of the Innermost Planet')
	#ax.set_xlabel('Orbital Period [days]')
	ax.set_ylabel('Planet Counts')

	#ax.set_xscale('log')
	#ax.set_xlim(epos.xtrim)
	helpers.set_axis_distance(ax, epos, Trim=True)
		
	#bins= np.geomspace(*epos.xtrim)
	bins= epos.MC_xvar

	#pdf= regression.sliding_window_log(P, None, xgrid) #, width=2. )
	#ax3.plot(xgrid, pdf, ls='-', marker='', color=clrs[k % 4],label='{} x{:.3f}'.format(sg['name'], sg['weight']))
				
	# MC data
	if MC or MCMC:
		ss=epos.synthetic_survey
		if MCMC:
			for ss in epos.ss_sample:
				ax.hist(ss['multi']['Pinner'], bins=bins, 
						color='b', alpha=0.1, histtype='step')
						
		else:
			ax.hist(ss['multi']['Pinner'], bins=bins, 
					color=color, histtype='stepfilled', label='Simulated') #epos.name)
			if hasattr(epos,'pfm'):
				ax.hist(epos.pfm['Pin'], bins=bins, 
					color='C7', histtype='step', label='Input', ls=':') #epos.name)

			# Solar system analologs (from dr25_solarsystem.py)
	# 		Pcut=45
	# 		Pcut=130
	# 		print 'P_in > {} days:'.format(Pcut)
	# 		print '  obs: {}'.format((epos.obs_zoom['multi']['Pinner']>Pcut).sum())
	# 		print '  sim: {}'.format((ss['multi']['Pinner']>Pcut).sum())
	# 		print ''

		#if False:
		if hasattr(epos, 'ss_extra'):
			# advance color cycle
			# ax._get_lines.get_next_color()
			ax.plot([], [])
			ax.plot([], [])
		
			# plot extra epos runs
			for ss in epos.ss_extra:
				ax.hist(ss['multi']['Pinner'], bins=bins, 
					histtype='step', label=ss['name'])
							
		if N:
			# Innermost is nth planet
			#ax.hist(ss['multi']['PN'][0], bins=bins, 
			#		ec='k', histtype='step', label='actual inner planet')
			# Not actual inner planet
			ax.hist(np.concatenate(ss['multi']['PN'][1:]), bins=bins, 
					color='r', histtype='stepfilled', label='Not Inner Planet')
		else:
			ax.hist(epos.obs_zoom['multi']['Pinner'], bins=bins, 
				ec='C3', histtype='step', label='Kepler', ls='--')

	else:
		# observed all
		ax.hist(epos.multi['Pinner'], bins=bins, color='0.7', label='Kepler all')

	''' Initial distribution or zoomed observations '''
	if Input and epos.Parametric:
		Pingrid= np.geomspace(*epos.xtrim)
		_, _, pdf0_X, _= draw_PR(epos, Init=True, ybin=epos.yzoom, xgrid=Pingrid)
		norm= 0.95* ax.get_ylim()[1] / max(pdf0_X)
		ax.plot(Pingrid, pdf0_X*norm, marker='',ls='-',color='r',label='Intrinsic')
				
	
	prefix= 'output' if MC else 'survey'
	suffix= '.index' if N else '' 
	suffix= '.input' if Input else suffix
	if MCMC: prefix= 'mcmc'

	if MC: ax.legend(loc='upper right', shadow=False, prop={'size':14}, numpoints=1)
		
	helpers.save(plt, '{}{}/innerperiod{}'.format(epos.plotdir, prefix,suffix))	

def periodinner_cdf(epos, MC=False, color='C1'):
	# plot multiplicity
	f, ax = plt.subplots()
	ax.set_title('period innermost planet')
	ax.set_xlabel('Orbital Period [days]')
	ax.set_ylabel('CDF')

	ax.set_xscale('log')
	ax.set_xlim(epos.xtrim)

	ax.set_ylim(-0.05, 1.05)
		
	bins= np.logspace(*np.log10(epos.xtrim))

	#pdf= regression.sliding_window_log(P, None, xgrid) #, width=2. )
	#ax3.plot(xgrid, pdf, ls='-', marker='', color=clrs[k % 4],label='{} x{:.3f}'.format(sg['name'], sg['weight']))
				
	# MC data
	if MC:
		ss=epos.synthetic_survey
		Psort= np.sort(ss['multi']['Pinner'])
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf, color=color, label=epos.name)
	
		# Observed zoom
		Psort= np.sort(epos.obs_zoom['multi']['Pinner'])
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf, color='C3', label='Kepler', ls='--')
	else:
		# observed all
		Psort= np.sort(epos.multi['Pinner'])
		cdf= np.arange(Psort.size, dtype=float)/Psort.size
		ax.plot(Psort, cdf ,color='gray' if MC else 'k', label='Kepler all')	
	
	prefix= 'output' if MC else 'survey'

	if MC: ax.legend(loc='lower right', shadow=False, prop={'size':14}, numpoints=1)
		
	helpers.save(plt, '{}{}/innerperiod.cdf'.format(epos.plotdir, prefix))	

''' Plot planet population '''

def inner(epos):
	'''
	Plot inner planet
	'''
	# plot multiplicity
	f, ax = plt.subplots()
	pop=epos.population
		
	ax.set_title('Simulated Systems')
	ax.set_xlabel('Orbital Period (days)')
	ax.set_ylabel('Fraction of stars') # 0-45 %

	#ax.set_ylim(-0.05, 1.05)
	ax.set_ylim(0, 1)
	ax.set_xlim(0.2, 730)
	ax.set_xscale('log')

	x=np.geomspace(0.2,730)
	# 	ax.fill_between(x, 0.0, 0.159, color='g', alpha=0.2, lw=0)
	# 	ax.fill_between(x, 0.159, 0.841, color='c', alpha=0.2, lw=0)
	# 	ax.fill_between(x, 0.841, 1, color='g', alpha=0.2, lw=0)
	ax.fill_between(x, 0.0, 0.023, color='g', alpha=0.2, lw=0)
	ax.fill_between(x, 0.023, 0.159, color='c', alpha=0.2, lw=0)
	ax.fill_between(x, 0.159, 0.841, color='y', alpha=0.2, lw=0)
	ax.fill_between(x, 0.841, 0.977, color='c', alpha=0.2, lw=0)
	ax.fill_between(x, 0.977, 1, color='g', alpha=0.2, lw=0)
	ax.axhline(0.5, color='k', lw=1)
	
	xtext=0.21
	ax.text(xtext, 0.023, '$2\sigma$', va='center',size=10)
	ax.text(xtext, 0.159, '$1\sigma$', va='center',size=10)
	#ax.text(xtext, 0.5, 'mean', va='center',size=8)
	ax.text(xtext, 0.5+0.01, 'mean',size=10)
	ax.text(xtext, 0.841, '$1\sigma$', va='center',size=10)
	ax.text(xtext, 0.977, '$2\sigma$', va='center',size=10)

	# BG all planets?
	# 	ax.plot(pop['P'], pop['order'],
	# 		ls='', marker='.', mew=0, ms=3, color='0.8')

	# all multis
	key='single'
	ax.plot(pop[key]['P'], pop[key]['order'],
		ls='', marker='.', mew=0, ms=3, color='b')

	key='multi'
	ax.plot(pop[key]['P'], pop[key]['order'],
		ls='', marker='.', mew=0, ms=3, color='purple')
	
	# 	for key in ['system','single','multi']:
	# 	ax.plot(pop['P'], pop['order'],
	# 		ls='', marker='.', mew=0, ms=3, color='gray')

	# Solar System
	#ax.plot([0.95]*4,[])
	yss= 0.9
	#yss= 0.1
	ax.text(88, yss, 'M', ha='center', va='center', color='r',size=8)
	ax.text(225, yss, 'V', ha='center', va='center', color='r',size=8)
	ax.text(365, yss, 'E', ha='center', va='center', color='r',size=8)
	ax.text(687, yss,'M', ha='center', va='center', color='r',size=8)

	helpers.save(plt, '{}/population/inner_test'.format(epos.plotdir))

def polar(epos):
	'''
	Plot planet populations as a half circle (not quite working) 
	'''
	# plot multiplicity
	f, axlist = plt.subplots(2,2, subplot_kw=dict(projection='polar'), figsize=(10,8))
	pop=epos.population
	
	# ticks not showing on log plot
	Bugged=True

	for ax, key in zip([axlist[0,0],axlist[0,1],axlist[1,0]],['system','single','multi']):
		ax.set_title(key)
		ax.set_xlabel('fraction')
		ax.set_ylabel('P [days]')

		ax.set_xlim(-0.05, 1.05)
		ax.set_xticks(np.pi*np.linspace(0,1,5))

		try:
			ax.set_thetamin(0)
			ax.set_thetamax(180)
		except AttributeError:
			print 'Update pyplot'
			raise
		
		if Bugged:
			ax.set_ylim(-1,3)
			#ax.set_yticks([0,1,2,3])
			ax.set_yticks([1])
			ax.set_yticklabels(['10'])
			ax.plot(pop[key]['order']*np.pi, np.log10(pop[key]['P']),
					ls='', marker='.', mew=0, ms=3, color='k')
			
		else:
			ax.set_ylim(0.1, 1000) # 7.
			ax.set_yscale('log')
			ax.set_yticks([1,10,100,1000])
			#ax.set_yticks([10])

			#ax.set_rlim(0.1,1000)
			#ax.set_rscale('log')
		
			#ax.set_rgrids([1, 10, 100, 1000])
			#ax.set_rticks([1, 10, 100, 1000])
				
			ax.plot(pop[key]['order']*np.pi, pop[key]['P'],
					ls='', marker='.', mew=0, ms=3, color='k')
		
		# nope
		#ax.set_yscale('log')
		#ax.set_yticks([1, 10, 100, 1000])
		

	ax4=axlist[1,1]
	ax4.set_title('all')
	ax4.set_xlabel('fraction')
	ax4.set_ylabel('P [days]')

	ax4.set_xlim(-0.05, 1.05)
	ax4.set_ylim(0.1, 1000) # 7.
	ax4.set_yscale('log')
	
	try:
		ax4.set_thetamin(0)
		ax4.set_thetamax(180)
	except AttributeError:
		print 'Update pyplot'
		raise
	
	ax4.plot(pop['order']*np.pi, pop['P'],
		ls='', marker='.', mew=0, ms=3, color='gray')

	helpers.save(plt, '{}/population/polar_test'.format(epos.plotdir))



import numpy as np
import matplotlib.pyplot as plt
import helpers
from EPOS import cgs
	
def massradius(epos, MC=False, Log=False, color='C1', Mlim=[0,20], Rlim=[0,7], rho=None):
	assert epos.MassRadius
	# plot R(M)
	f, ax = plt.subplots()
	ax.set_title('mass-radius relation + dispersion')
	ax.set_xlabel(r'Planet Mass [M$_\bigoplus$]')
	ax.set_ylabel(r'Planet Radius [R$_\bigoplus$]')

	if Log:
		ax.set_xlim(epos.masslimits)
		ax.set_ylim(0.1, 30.) # 7.	
		ax.set_xscale('log')
		ax.set_yscale('log')
	else:
		ax.set_xlim(*Mlim) # 20.
		ax.set_ylim(*Rlim) # 7.
	
	# MC data
	if MC:
		ss=epos.synthetic_survey
		ax.plot(ss['M'], ss['R'], ls='', marker='.', mew=0, ms=5.0, color=color, zorder=1)
		
	xM= np.logspace(*np.log10(ax.get_xlim())) if Log else np.linspace(*ax.get_xlim()) #np.max(tr['M']))
	xR, dispersion= epos.MR(xM)
	#ax.plot(xM, epos.MR['fRock'](xM), ls='-', marker='', color='r', label='Rocky constraint', zorder=0)
	#xGas= epos.MR['fGas'](xM)
	#dispersion= epos.MR['dispersion']
	ax.plot(xM, xR, ls='-', marker='', color='b', label=epos.MR_label)
	ax.plot(xM, xR-dispersion, ls='--', marker='', color='b')
	ax.plot(xM, xR+dispersion, ls='--', marker='', color='b')

	# constant density used in N-body
	if rho is not None:
		rhoR= (xM*cgs.Mearth / (4./3.*np.pi*rho))**(1./3.) / cgs.Rearth
		ax.plot(xM, rhoR, ls='-', color='0.5', marker='', 
			label=r'$\rho={:.1f}\,g\,cm^{{-3}}$'.format(rho), zorder=-1)

	ax.legend(loc='lower right', shadow=False, prop={'size':14}, numpoints=1)
	
	suffix= 'MC' if MC else 'input'
	suffix+='.log' if Log else ''	
	
	helpers.save(plt, '{}/massradius/{}'.format(epos.plotdir,suffix))		



import numpy as np
from scipy import interpolate
import multiprocessing
from functools import partial

import shapely.geometry

from EPOS.population import periodradius
import EPOS.analytics

def all(epos, BinnedMetric=False):
	if hasattr(epos,'occurrence'):
		planets(epos)

	if hasattr(epos,'pfm'):
		models(epos)

	if hasattr(epos,'occurrence'):
		if 'bin' in epos.occurrence:
			binned(epos)
		if 'poly' in epos.occurrence:
			poly_binned(epos)
		if 'xzoom' in epos.occurrence and 'yzoom' in epos.occurrence:
			zoomed(epos)
		
		# posterior per bin
		if epos.Prep and epos.Parametric and (not epos.Multi) \
				and 'bin' in epos.occurrence:
			parametric(epos)

			''' BIC/AIC with chi^2'''
			if BinnedMetric and 'yzoom' in epos.occurrence or 'xzoom' in epos.occurrence:
				print '\n  Binned occurrence rate metrics'
				binned_metric(epos)
	else:
		print 'No bins for calculating occurrence rate, did you use epos.set_bins() ?'
	
def planets(epos, Log=False):
	''' 
	Calculate the intrinsic occurrence of each observed planet by interpolating the survey completeness
	'''
	if not epos.Range: epos.set_ranges()
	
	''' Interpolate occurrence for each planet (on log scale) '''
	focc= epos.occurrence
	
	print '\nInterpolating planet occurrence'
	#with np.errstate(divide='ignore'): 
	#print '{}x{}=?={}'.format(epos.eff_xvar.shape, epos.eff_yvar.shape, epos.completeness.shape)
	
	completeness= _interpolate_occ(epos, epos.obs_xvar, epos.obs_yvar, Log=Log)

	focc['planet']={}
	focc['planet']['xvar']= epos.obs_xvar
	focc['planet']['yvar']= epos.obs_yvar	
	focc['planet']['completeness']= completeness
	focc['planet']['occ']= 1./completeness/epos.nstars
	#print epos.planet_occurrence

def models(epos, Log=False):
	'''
	Interpolate the survey completeness for each modeled planet
	'''
	if not epos.Range: epos.set_ranges()

	if not hasattr(epos,'occurrence'):
		epos.occurrence={}
	
	''' Interpolate occurrence for each planet (on log scale) '''
	focc= epos.occurrence
	print '\nInterpolating model planet occurrence'

	pfm=epos.pfm
	if not 'R' in pfm:
		pfm['R'], _= epos.MR(pfm['M'])
	
	if Log:
		# does not seem to make a big difference
		pl_comp_log= interpolate.RectBivariateSpline(np.log10(epos.eff_xvar), 
			np.log10(epos.eff_yvar), np.log10(epos.completeness))
		completeness= 10.**pl_comp_log(np.log10(pfm['P']), np.log10(pfm['R']), grid=False) 
		#print completeness
	else:
		# if zeros in detection efficiency?
		pl_comp= interpolate.RectBivariateSpline(epos.eff_xvar, 
			epos.eff_yvar, epos.completeness)
		completeness= pl_comp(pfm['P'], pfm['R'], grid=False)
		
	focc['model']={}
	focc['model']['completeness']= completeness
	focc['model']['eta']= epos.fitpars.getpps() if hasattr(epos, 'fitpars') else 1.
	#focc['model']['occ']= focc['model']['eta']/completeness/epos.nstars
	#print epos.planet_occurrence

def binned(epos, Log=False):
	'''
	Calculate the planet occurrence rate per bin using inverse detection efficiency.
	(these are the bins from the detection efficiency grid)
	'''	
	focc= epos.occurrence

	# set y bin
	if epos.MassRadius:
		focc['bin']['y']= epos.MR(focc['bin']['y in'])[0]
		for i, ybin in enumerate(focc['bin']['y']):
			if ybin[0]>ybin[-1]: 
				print '\nWarning: observed bins are not increasing in size'
				print '   {} > {}'.format(ybin[0],ybin[-1])
				focc['bin']['y'][i]= focc['bin']['y'][i][::-1] 
	else:
		focc['bin']['y']= focc['bin']['y in']

	print '\n  Observed Planets'
	# calc n, i, occ, err, xc, yc, dlnx, dlny from x & y
	_occ_per_bin(epos, focc['bin'], Log=Log)
	
	if 'model' in focc:
		eta= focc['model']['eta']
		print '\n  Modeled planets, eta= {:.3g}'.format(eta)
		_model_occ_per_bin(epos, focc['bin'], focc['model'], weights=eta/epos.pfm['ns'])

def poly_binned(epos, Log=False):
	'''
	Calculate the planet occurrence rate per bin using inverse detection efficiency.
	(these are the polygonic bins)
	'''	
	focc= epos.occurrence

	# set y bin
	if epos.MassRadius:
		assert False, 'implement polygons for planet mass'

	print '\n  Observed Planets'
	# calc n, occ, err, dlnxy 
	_occ_per_polygon(epos, focc['poly'], Log=Log)
	
	if 'model' in focc:
		eta= focc['model']['eta']
		print '\n  Modeled planets, eta= {:.3g}'.format(eta)
		_model_occ_per_polygon(epos, focc['poly'], focc['model'], 
			weights=eta/epos.pfm['ns'])

def zoomed(epos, TestScore= True):
	''' Occurrence (inverse detection efficiency) along x,y axis.'''
	focc= epos.occurrence

	print '\n  x zoom bins'
	_occ_per_bin(epos, focc['xzoom'])
	print '\n  y zoom bins'
	_occ_per_bin(epos, focc['yzoom'])

def _occ_per_bin(epos, foccbin, Log=False):
	''' Planet occurrence (inverse detection efficiency) per bin '''	
	_occ, _n, _inbin, _xc, _yc, _dlnx, _dlny=[],[],[],[],[],[],[]
	
	for xbin, ybin in zip(foccbin['x'],foccbin['y']):
		inbinx= (xbin[0]<=epos.obs_xvar) & (epos.obs_xvar<xbin[1])
		inbiny= (ybin[0]<=epos.obs_yvar) & (epos.obs_yvar<ybin[1])
		inbin= inbinx & inbiny
		
		_inbin.append(inbin)
		_n.append(inbin.sum())
		_occ.append(epos.occurrence['planet']['occ'][inbin].sum())
		
		print '  x: [{:.3g},{:.3g}], y: [{:.2g},{:.2g}], n={}, occ={:.2g}'.format(
			xbin[0],xbin[-1], ybin[0],ybin[-1], _n[-1], _occ[-1])
		
		_xc.append(np.sqrt(xbin[0])*np.sqrt(xbin[-1]) )
		_yc.append(np.sqrt(ybin[0])*np.sqrt(ybin[-1]) )
		_dlnx.append(np.log(xbin[-1]/xbin[0]))
		_dlny.append(np.log(ybin[-1]/ybin[0]))

	foccbin['xc']= np.array(_xc)
	foccbin['yc']= np.array(_yc)
	foccbin['dlnx']= np.array(_dlnx)
	foccbin['dlny']= np.array(_dlny)

	foccbin['n']= np.array(_n)
	foccbin['i']= np.array(_inbin)
	foccbin['occ']= np.array(_occ)
	foccbin['err']= foccbin['occ']/np.where(
							foccbin['n']>0,np.sqrt(foccbin['n']),1.)
	isUL= foccbin['n']==0
	if isUL.sum()>0:
		completeness_UL= _interpolate_occ(epos, foccbin['xc'][isUL], foccbin['yc'][isUL], Log=Log)
		foccbin['err'][isUL]= 1./completeness_UL/epos.nstars 

def _occ_per_polygon(epos, foccpoly, Log=False):
	''' Planet occurrence (inverse detection efficiency) per bin '''	
	_occ, _n, _inbin, _xc, _yc, _dlnxy=[],[],[],[], [], []
	
	xyp= np.hstack([epos.obs_xvar[:,None],epos.obs_yvar[:,None]])

	for xycoords in foccpoly['coords']:
		shp=shapely.geometry.polygon.Polygon(xycoords)
		shp_log=shapely.geometry.polygon.Polygon(np.log(xycoords))

		points= map(shapely.geometry.Point, xyp)
		inpoly= map(shp.contains, points)

		logcenter= shp_log.centroid
		
		_inbin.append(inpoly)
		_n.append(np.sum(inpoly))
		_occ.append(epos.occurrence['planet']['occ'][inpoly].sum())
		_dlnxy.append(np.log(shp_log.area))

		_xc.append(np.exp(logcenter.x))
		_yc.append(np.exp(logcenter.y))

		print '  poly, n={}, occ={:.2g}, area={:.2g}'.format(_n[-1], _occ[-1], _dlnxy[-1])
		print '    xc= {:.2g}, yc={:.2g}'.format(_xc[-1], _yc[-1])
	
	foccpoly['xc']= np.array(_xc)
	foccpoly['yc']= np.array(_yc)	
	foccpoly['dlnxy']= np.array(_dlnxy)

	foccpoly['n']= np.array(_n)
	foccpoly['i']= np.array(_inbin)

	foccpoly['occ']= np.array(_occ)
	foccpoly['err']= foccpoly['occ']/np.where(
							foccpoly['n']>0,np.sqrt(foccpoly['n']),1.)
	isUL= foccpoly['n']==0
	if isUL.sum()>0:
		completeness_UL= _interpolate_occ(epos, foccpoly['xc'][isUL], foccpoly['yc'][isUL], Log=Log)
		foccpoly['err'][isUL]= 1./completeness_UL/epos.nstars 

def _model_occ_per_bin(epos, foccbin, foccmodel, weights=None):	
	_occ, _n, _inbin, _xc, _yc, _dlnx, _dlny=[],[],[],[],[],[],[]
	
	for xbin, ybin in zip(foccbin['x'],foccbin['y']):
		inbinx= (xbin[0]<=epos.pfm['P']) & (epos.pfm['P']<xbin[1])
		inbiny= (ybin[0]<=epos.pfm['R']) & (epos.pfm['R']<ybin[1])
		inbin= inbinx & inbiny
		
		_inbin.append(inbin)
		_n.append(inbin.sum())
		_occ.append(weights* inbin.sum())
		
		print '  x: [{:.3g},{:.3g}], y: [{:.2g},{:.2g}], n={}, occ={:.2g}'.format(
			xbin[0],xbin[-1], ybin[0],ybin[-1], _n[-1], _occ[-1])
		
		_xc.append(np.sqrt(xbin[0])*np.sqrt(xbin[-1]) )
		_yc.append(np.sqrt(ybin[0])*np.sqrt(ybin[-1]) )
		_dlnx.append(np.log(xbin[-1]/xbin[0]))
		_dlny.append(np.log(ybin[-1]/ybin[0]))

	_foccbin= foccmodel['bin']= {}
	_foccbin['n']= np.array(_n)
	_foccbin['i']= np.array(_inbin)
	_foccbin['occ']= np.array(_occ)
	_foccbin['err']= _foccbin['occ']/np.where(
							_foccbin['n']>0,np.sqrt(_foccbin['n']),1.)
	
	_foccbin['xc']= np.array(_xc)
	_foccbin['yc']= np.array(_yc)
	_foccbin['dlnx']= np.array(_dlnx)
	_foccbin['dlny']= np.array(_dlny)
	
	_foccbin['x']= foccbin['x']
	_foccbin['y']= foccbin['y']

def _model_occ_per_polygon(epos, foccpoly, foccmodel, weights=None):	
	_occ, _n, _inbin, _xc, _yc, _dlnxy=[],[],[],[], [], []
	
	xyp= np.hstack([epos.pfm['P'][:,None],epos.pfm['R'][:,None]])

	for xycoords in foccpoly['coords']:
		shp=shapely.geometry.polygon.Polygon(xycoords)
		shp_log=shapely.geometry.polygon.Polygon(np.log(xycoords))

		points= map(shapely.geometry.Point, xyp)
		inpoly= map(shp.contains, points)

		logcenter= shp_log.centroid
		
		_inbin.append(inpoly)
		_n.append(np.sum(inpoly))
		_occ.append(weights* np.sum(inpoly))
		_dlnxy.append(np.log(shp_log.area))

		_xc.append(np.exp(logcenter.x))
		_yc.append(np.exp(logcenter.y))

		print '  poly, n={}, occ={:.2g}, area={:.2g}'.format(_n[-1], _occ[-1], _dlnxy[-1])
		print '    xc= {:.2g}, yc={:.2g}'.format(_xc[-1], _yc[-1])
		
	_foccpoly= foccmodel['poly']= {}
	_foccpoly['n']= np.array(_n)
	_foccpoly['i']= np.array(_inbin)
	_foccpoly['occ']= np.array(_occ)
	_foccpoly['err']= _foccpoly['occ']/np.where(
							_foccpoly['n']>0,np.sqrt(_foccpoly['n']),1.)
	
	_foccpoly['xc']= np.array(_xc)
	_foccpoly['yc']= np.array(_yc)
	_foccpoly['dlnxy']= np.array(_dlnxy)
	
	_foccpoly['coords']= foccpoly['coords']

def parametric(epos):
	''' Calculates the occurrence rate per bin from the parametric model, 
	with uncertainties if samples from an MCMC chain are available
	'''
	assert epos.Prep and epos.Parametric and (not epos.Multi)
	focc= epos.occurrence
	
	''' loop over all pre-defined bins '''
	print '\n  posterior per bin'
	xbins= focc['bin']['x']
	ybins= focc['bin']['y in']
	eta, gamma, area, pos, sigp, sign= _posterior_per_bin(epos, xbins, ybins, Verbose=True)

	focc['bin']['area']= np.array(area)
	focc['bin']['gamma0']= np.array(gamma)
	focc['bin']['eta0']= np.array(eta)	
	
	if hasattr(epos, 'samples'):
		focc['bin']['gamma']= np.array(pos)
		focc['bin']['gamma+']= np.array(sigp)
		focc['bin']['gamma-']= np.array(sign)
		focc['bin']['eta']= focc['bin']['gamma']*focc['bin']['area']
		focc['bin']['eta+']= focc['bin']['gamma+']*focc['bin']['area']
		focc['bin']['eta-']= focc['bin']['gamma-']*focc['bin']['area']

	''' bin for normalization '''
	if hasattr(epos.fitpars, 'normkeyx') and hasattr(epos.fitpars, 'normkeyy'):
		print '\n  normalization per unit of ln area'
		dw= 1.01
		xnorm= epos.fitpars.get(epos.fitpars.normkeyx)
		ynorm= epos.fitpars.get(epos.fitpars.normkeyy)
		xbin= [xnorm/dw, xnorm*dw]
		ybin= [ynorm/dw, ynorm*dw]
		eta, gamma, _, gamma_fit, gamma_p, gamma_n=  _posterior_per_bin(epos, [xbin],[ybin])
		print '  x={:.2g}, y={:.2g}, gamma= {:.2g}'.format(xnorm, ynorm, gamma[0])
		if len(gamma_fit)>0:
			print '  gamma= {:.2g} +{:.2g} - {:.2g}'.format(gamma_fit[0], gamma_p[0], gamma_n[0])

def _interpolate_occ(epos, out_xvar, out_yvar, Log=False):
	if Log:
		# does not seem to make a big difference
		pl_comp_log= interpolate.RectBivariateSpline(np.log10(epos.eff_xvar), 
			np.log10(epos.eff_yvar), np.log10(epos.completeness))
		out_completeness= 10.**pl_comp_log(np.log10(out_xvar), np.log10(out_yvar), grid=False) 
		#print completeness
	else:
		# if zeros in detection efficiency?
		pl_comp= interpolate.RectBivariateSpline(epos.eff_xvar, epos.eff_yvar, epos.completeness)
		out_completeness= pl_comp(out_xvar, out_yvar, grid=False)

	return out_completeness

def _posterior(epos, sample, xbin, ybin):
	_, pdf, _, _= periodradius(epos, fpara=sample, xbin=xbin, ybin=ybin)
	return np.average(pdf)

def _posterior_per_bin(epos, xbins, ybins, Verbose=True):
	eta, gamma, area= [], [], []
	pos, sigp, sign=[], [], []
	
	for xbin, ybin in zip(xbins,ybins):
		area.append(np.log(xbin[1]/xbin[0])*np.log(ybin[1]/ybin[0]))
		_, pdf, _, _= periodradius(epos, Init=True, xbin=xbin, ybin=ybin)
		gamma.append(np.average(pdf))
		eta.append(gamma[-1]*area[-1])

		if Verbose:
			print '  x: [{:.3g},{:.3g}], y: [{:.2g},{:.2g}], area={:.2f}, eta_0={:.2g}'.format(
				xbin[0],xbin[-1], ybin[0],ybin[-1], area[-1], eta[-1])

		''' Posterior?'''
		if hasattr(epos, 'samples'):
			posterior= []
			if epos.Parallel:
				pool = multiprocessing.Pool()
				one_arg_func= partial(_posterior, epos, xbin=xbin, ybin=ybin)
				posterior= pool.map(one_arg_func, epos.samples)
			else:
				for sample in epos.samples:
					_, pdf, _, _= periodradius(epos, fpara=sample, xbin=xbin, ybin=ybin)
					posterior.append(np.average(pdf))

			#pos= np.percentile(posterior, [16, 50, 84])
			perc= np.percentile(posterior, [2.3, 15.9, 50., 84.1, 97.7])
			pos.append(perc[2])
			sigp.append(perc[3]-perc[2])
			sign.append(perc[2]-perc[1])
			sig2p= (perc[4]-perc[2])
			sig2n= (perc[2]-perc[0])
		
			if Verbose:
				print '  gamma= {:.1%} +{:.1%} -{:.1%}'.format(pos[-1],sigp[-1],sign[-1])
				print '  eta= {:.1%} +{:.1%} -{:.1%}'.format(
					pos[-1]*area[-1],sigp[-1]*area[-1],sign[-1]*area[-1])
				#print '  gamma 2sig= {:.1%} +{:.1%} -{:.1%}'.format(_pos[-1],sig2p,sig2n)

	return eta, gamma, area, pos, sigp, sign
	

def binned_metric(epos):
	''' occurrence for one y bin, along x axis'''
	if 'yzoom' in epos.occurrence:
		xc= epos.occurrence['yzoom']['xc']
		occ_obs= epos.occurrence['yzoom']['occ'] / epos.occurrence['yzoom']['dlnx']
		occ_err= epos.occurrence['yzoom']['err'] / epos.occurrence['yzoom']['dlnx']

		_, _, pdf_X, _= periodradius(epos, ybin=epos.yzoom, xgrid= epos.occurrence['yzoom']['xc'])
		#for args in zip(xc, occ_obs, occ_err, pdf_X):
			#print 'xc= {:.2g}, occ={:.2g}+-{:.2g}, model= {:.2g}'.format(*args)

		squared_residuals= ((occ_obs-pdf_X)/occ_err)**2.

		zoom= (epos.xzoom[0] < xc) & (xc < epos.xzoom[-1])

		rss= np.sum(squared_residuals[zoom])
		nbins= zoom.sum()
		kfree= epos.fitpars.get_kfree()
		dof= nbins-kfree # kfree includes normalization parameter
		chi2= rss / dof

		bic= EPOS.analytics.bic_rss(rss, kfree, nbins)
		aic= EPOS.analytics.aic_rss(rss, kfree, nbins)
		aic_c= EPOS.analytics.aic_c_rss(rss, kfree, nbins)

		print '  x: (n={}, k={})'.format(nbins, kfree)
		print '    chi^2= {:.1f}, reduced= {:.1f}'.format(rss, chi2)
		print '    bic= {:.1f}'.format(bic)
		print '    aic= {:.1f}, AICc= {:.1f}'.format(aic, aic_c)

	if 'xzoom' in epos.occurrence:
		yc= epos.occurrence['xzoom']['yc']
		occ_obs= epos.occurrence['xzoom']['occ'] / epos.occurrence['xzoom']['dlny']
		occ_err= epos.occurrence['xzoom']['err'] / epos.occurrence['xzoom']['dlny']

		_, _, _, pdf_Y= periodradius(epos, xbin=epos.xzoom, ygrid= epos.occurrence['xzoom']['yc'])
		#for args in zip(yc, occ_obs, occ_err, pdf_Y):
			#print 'yc= {:.2g}, occ={:.2g}+-{:.2g}, model= {:.2g}'.format(*args)

		squared_residuals= ((occ_obs-pdf_Y)/occ_err)**2.

		zoom= (epos.yzoom[0] < yc) & (yc < epos.yzoom[-1])

		rss= np.sum(squared_residuals[zoom])
		nbins= zoom.sum()
		kfree= epos.fitpars.get_kfree()
		dof= nbins-kfree # kfree includes normalization parameter
		chi2= rss / dof

		bic= EPOS.analytics.bic_rss(rss, kfree, nbins)
		aic= EPOS.analytics.aic_rss(rss, kfree, nbins)
		aic_c= EPOS.analytics.aic_c_rss(rss, kfree, nbins)

		print '  y: (n={}, k={})'.format(nbins, kfree)
		print '    chi^2= {:.1f}, reduced= {:.1f}'.format(rss, chi2)
		print '    bic= {:.1f}'.format(bic)
		print '    aic= {:.1f}, AICc= {:.1f}'.format(aic, aic_c)



import numpy as np

def save(dict,fname,tag='KID'):
	import numpy as np
	
	Mkeys=dict.keys()
	Munits=[]
	for key in Mkeys:
		if type(dict[key][0])==type(np.array([0.])[0]): Munits.append('float')
		elif type(dict[key][0])==type(np.array([0])[0]): Munits.append('int')
		elif type(dict[key][0])==type(np.array([''])[0]): Munits.append('string')
		elif type(dict[key][0])==type(np.array([True])[0]): Munits.append('bool')
		else: print 'no matching type for {} ({})'.format(key,type(dict[key][0]))
					
	with open(fname, 'w') as f:
		f.write(",".join(Mkeys)+'\n')
		f.write(",".join(Munits)+'\n')
		#for i in range(0,len(dict[tag])):
		for i in range(len(dict[Mkeys[0]])): 
			f.write(",".join([str(dict[ikey][i]) for ikey in Mkeys])+'\n')
			
def load(fname):
	import numpy as np
	import csv
	
	with open(fname, 'rb') as f:
		reader = csv.reader(f)
		Mkeys=reader.next()
		Munits=reader.next()
		dict={}
		for key in Mkeys: dict[key]=[]
		for row in reader:
			for j in range(0,len(Mkeys)):
				dict[Mkeys[j]].append(row[j])
		npdict={}
		for j,key in enumerate(Mkeys):
			if Munits[j]=='float': npdict[key]=np.array(dict[key],dtype=float)
			elif Munits[j]=='int': npdict[key]=np.array(dict[key],dtype=int)
			elif Munits[j]=='bool': npdict[key]=(np.array(dict[key])=='True')
			elif Munits[j]=='string': npdict[key]=np.array(dict[key])
			else: print 'no matching type for {} ({})'.format(key,type(dict[key][0]))
			
	return npdict
	
# 	updates fields in dictionary 1 with values from dictionary 2
#	The field being matched is the first entry in field1
# 	Fields updated can have different names, in this case supply field2 
# 	needs a check if field exists
def update(dict1,dict2,field1=[],field2=[],verbose=False,report_missing=False):
	import numpy as np
	
	if len(field1) < 2: print 'update.dictionary: need at least two fields!!'
	if len(field2) == 0: field2=field1
	if len(field1) != len(field2): print 'update.dictionary: fields need to be same length!!'


#	np.set_printoptions(threshold='nan')
		
	print "  matching field '{}' to '{}'".format(field1[0],field2[0])
	array1=dict1[field1[0]]
	array2=dict2[field2[0]]
	match1=[]  # get matching indices
	match2=[]
	for j in range(0,len(array2)):
		index=np.where(array2[j] == array1)
		if len(index[0]) == 0: 
			if verbose: print "field '{}' with value {} has no matches!".format(field2[0],array2[j])
			if report_missing: print "field '{}' with value {} has no matches!".format(field2[0],array2[j])
		elif len(index[0]) == 1: 
			match1.append(int(index[0]))
			match2.append(j)
			if verbose: print '	Couple {} to {}   (Matched {} == {})'.format(match1[-1],match2[-1],array1[match1[-1]],array2[match2[-1]])
		else: 
			match1.extend([int(i) for i in index[0]])
			match2.extend([j]*len(index[0]))
			if verbose:
				print 'update.dictionary: more than one match (???)'
				print '	Couple {} to {}'.format(match1[-1],match2[-1])
				for ii in range(0,len(index[0])):
					print '   (Matched {} == {})'.format(array1[match1[-1-ii]],array2[match2[-1-ii]])

	if len(match1) != len(match2): print 'update.dictionary: something went wrong!'
	print '  {} matching items out of {} and {}'.format(len(match1),len(array1),len(array2))
	
	for i in range(1,len(field1)):
		if field1[i] in dict1.keys():
			if verbose: print
			print "   copying field '{}' to '{}'".format(field2[i],field1[i])
			if verbose:
				for j in range(0,len(match1)): print '	Replace {} by {}'.format(dict1[field1[i]][match1[j]],dict2[field2[i]][match2[j]])
			dict1[field1[i]][match1]=dict2[field2[i]][match2]
		else:
			if (len(match1) == len(match2)):
				print "   copying field '{}' to new field '{}'".format(field2[i],field1[i])
				dict1[field1[i]]=np.zeros(len(match1),dtype=dict2[field2[i]].dtype)
				dict1[field1[i]][match1]=dict2[field2[i]][match2]
			else:
				print "  !! no field '{}' to merge into !!".format(field1[i])
		
	print
	
def remove_nonmatching_elements(dict1,dict2,field='KID',report_missing=False, ReturnNonMatch=False):
	# remove elements from (dict1) that are not found in (field) from (dict2)
	# ReturnNonMatch actually keeps those elements
	import numpy as np
	
	match=[]
	if ReturnNonMatch: nonmatch= []
	for i,KID in enumerate(dict1[field]):
		index=np.where(KID == dict2[field])
		if len(index[0]) == 0:
			if ReturnNonMatch: nonmatch.append(i)
			if report_missing: print "field '{}' with value {} has no matches!".format(field,KID)
		else:
			match.append(i)
	
	if ReturnNonMatch:
		print '  {} elements, {} matching, {} nonmatching'.format(len(dict1[field]), len(match), len(nonmatch))
	
	dict={}
	tomatch=nonmatch if ReturnNonMatch else match
	for key in dict1.keys():
		dict[key]=dict1[key][tomatch]
	
	print '  removed {} elements, {} left'.format(len(dict1[field])-len(tomatch), len(tomatch))
	
	return dict
	
def remove_elements(dict1,TFarray,verbose=True):
	# remove elements from (dict1) that are True in (TFarray)
	import numpy as np
	
	remove=np.where(TFarray == True)
	keep=np.where(TFarray == False)
	
	dict={}
	for key in dict1.keys():
		if type(dict1[key])==np.ndarray and len(dict1[key]) == len(TFarray):
			dict[key]=dict1[key][keep]
		else:
			print "could not remove elements from '{}'".format(key)
			dict[key]=dict1[key]
	
	if verbose: print '  removed {} elements, {} left'.format(len(remove[0]), len(keep[0]))
	
	return dict
	
def subset(dict1, bin, lenkey='KID', verbose=True, report_missing=True):
	# keep only elements from (dict1) based on np.where bin
	# returns a COPY
	import numpy as np
	
	length= len(dict1[lenkey])
	dict={}
	for key in dict1.keys():
		if (type(dict1[key])==np.ndarray) and len(dict1[key]) == length:
			dict[key]=dict1[key][bin]
		else:
			if report_missing: print "could not remove elements from '{}'".format(key)
			dict[key]=dict1[key]
	
	if verbose: print '  extracted {} elements out of {}'.format(len(bin[0]), length)
	
	return dict

def concatenate(dict1, dict2, verbose=False):
	ret={}
	for key in dict1:
		#if key in dict2 and type(dict1[key]) is np.ndarray:
		if key in dict2:
			ret[key]= np.concatenate( [dict1[key], dict2[key]] )
			# print key
		#else: print 'not found in dict2: ' + key
	
	if verbose: print 'Merged keys: {}'.format(ret.keys())
	return ret
		


#! /usr/bin/env ipython
''' 
Test if EPOS can calculate occurrence rates

Plots should appear in the directory
png/test_5/occurrence/
'''

import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='test_5')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' define the parameteric distribution, here a power-law in radius and period '''
epos.set_parametric(EPOS.fitfunctions.powerlaw2D)
epos.fitpars.add('pps', 2.0, min=0)
epos.fitpars.add('P1',0.3, is2D=True)
epos.fitpars.add('P2',-0.2, dx=0.1, is2D=True)

''' define the simulated range (trim) and the range compared to observations (zoom) '''
epos.set_ranges(xtrim=[10,730],ytrim=[0.5,4.],xzoom=[20,300],yzoom=[0.7,3], Occ=True)

''' define bins where occurrence is calculated'''
epos.set_bins(xbins=[[20,300],[0.9*365,2.2*365]], ybins=[[0.7,3],[0.7,1.5]]) # eta_zoom, eta_earth

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' Calculate the occurrence rates '''
EPOS.occurrence.all(epos)

''' Adjust axes'''
epos.plotpars['textsize']= 12
epos.xtrim[1]= 1000

''' plot occurrence rates '''
EPOS.plot.occurrence.all(epos)


''' helpers '''
import os
from matplotlib.colors import Normalize
from matplotlib import cm
from matplotlib import gridspec

def set_axes(ax, epos, **args):
	set_axis_distance(ax, epos, **args)
	set_axis_size(ax, epos, **args)

def set_axis_distance(ax, epos, Trim=False, Eff=False, In=False, IsX=True):

	if IsX:
		ax.set_xlabel('Orbital Period [days]')
		if Trim:
			ax.set_xlim(epos.xtrim)
		elif Eff:
			ax.set_xlim(epos.eff_xlim)
		else:
			ax.set_xlim(epos.obs_xlim)
		
		ax.set_xscale('log')
		if hasattr(epos,'xticks'):
			ax.set_xticks(epos.xticks)
			ax.set_xticklabels(epos.xticklabels)
	
	else:
		ax.set_ylabel('Orbital Period [days]')
		if Trim:
			ax.set_ylim(epos.xtrim)
		elif Eff:
			ax.set_ylim(epos.eff_xlim)
		else:
			ax.set_ylim(epos.obs_xlim)
		
		ax.set_yscale('log')
		if hasattr(epos,'xticks'):
			ax.set_yticks(epos.xticks)
			ax.set_yticklabels(epos.xticklabels)

def set_axis_size(ax, epos, Trim=False, Eff=False, In=False, IsY=True):

	ticks=None
	if In:
		label= r'M [M$_\bigoplus$]'
		if hasattr(epos,'y_inticks'):
			ticks= epos.y_inticks
			ticklabels= epos.y_inticklabels
	elif epos.RV:
		label= r'M sin i [M$_\bigoplus$]'
		if hasattr(epos,'yticks'):
			ticks= epos.yticks
			ticklabels= epos.yticklabels
	else:
		label= r'Planet Radius [R$_\bigoplus$]'
		if hasattr(epos,'yticks'):
			ticks= epos.yticks
			ticklabels= epos.yticklabels

	if IsY:
		ax.set_ylabel(label)

		if Trim:
			ax.set_ylim(epos.in_ytrim if In else epos.ytrim)
		elif Eff:
			ax.set_ylim(epos.eff_ylim)
		else:
			ax.set_ylim(epos.obs_ylim)
		
		ax.set_yscale('log')
		if hasattr(epos,'yticks'):
		#if ticks is not None:
			ax.set_yticks(ticks)
			ax.set_yticklabels(ticklabels)
		
	else:
		ax.set_xlabel(label)

		if Trim:
			ax.set_xlim(epos.in_ytrim if In else epos.ytrim)
		elif Eff:
			ax.set_xlim(epos.eff_ylim)
		else:
			ax.set_xlim(epos.obs_ylim)
		
		ax.set_xscale('log')
		if hasattr(epos,'yticks'):
			ax.set_xticks(ticks)
			ax.set_xticklabels(ticklabels)

def make_panels(plt, Fancy=False):
	if Fancy:
		gs = gridspec.GridSpec(2, 2,
	                       width_ratios=[20,4],
	                       height_ratios=[3, 10]
	                       )
		f= plt.figure()
		f.subplots_adjust(wspace=0, hspace=0)

		axR = plt.subplot(gs[1, 1])
		axP = plt.subplot(gs[0, 0])

		ax = plt.subplot(gs[1, 0], sharex=axP, sharey=axR)	
		#axb = plt.subplot(gs[0, 2])	



		#axh.tick_params(direction='in', which='both', left=False, right=True, labelleft=False)
		#axh.yaxis.set_label_position('right')
		axR.axis('off')
		axP.axis('off')
		
		ax.tick_params(direction='out', which='both', top=False, right=False, 
			bottom=True, left=True)
		axP.tick_params(direction='out', which='both', top=False, right=False, 
			bottom=True, left=False)
		axR.tick_params(direction='out', which='both', top=False, right=False, 
			bottom=False, left=True)
	else:		
		gs = gridspec.GridSpec(2, 2,
	                       width_ratios=[4, 20],
	                       height_ratios=[10, 3]
	                       )
		f= plt.figure()
		f.subplots_adjust(wspace=0, hspace=0)
		
		ax = plt.subplot(gs[0, 1])	
		#axb = plt.subplot(gs[0, 2])	

		axR = plt.subplot(gs[0, 0], sharey=ax)
		axP = plt.subplot(gs[1, 1], sharex=ax)
		
		ax.tick_params(direction='in', which='both', top=True, right=True, 
			bottom=False, left=False)

	return f, (ax, axR, axP)

def make_panels_right(plt):
	gs = gridspec.GridSpec(2, 2,
                       width_ratios=[20, 4],
                       height_ratios=[10, 3]
                       )
	f= plt.figure()
	f.subplots_adjust(wspace=0, hspace=0)
	
	ax = plt.subplot(gs[0, 0])	
	#axb = plt.subplot(gs[0, 2])	

	axR = plt.subplot(gs[0, 1], sharey=ax)
	axP = plt.subplot(gs[1, 0], sharex=ax)

	ax.tick_params(direction='in', which='both', top=True)
	ax.tick_params(direction='out', which='both', left=True, top=False)
	
	axR.yaxis.tick_right()
	#axR.tick_params(direction='out', which='both', right=True, left=False)
	#axR.yaxis.set_label_position('right')
			
	return f, (ax, axR, axP)

def make_panels_clrbar(plt):
	gs = gridspec.GridSpec(2, 3,
                       width_ratios=[4, 20, 1],
                       height_ratios=[10, 3]
                       )
	f= plt.figure()
	f.subplots_adjust(wspace=0, hspace=0)
	
	ax = plt.subplot(gs[0, 1])	
	axb = plt.subplot(gs[0, 2])	

	axR = plt.subplot(gs[0, 0], sharey=ax)
	axP = plt.subplot(gs[1, 1], sharex=ax)

	return f, (ax, axb, axR, axP)

def set_pyplot_defaults():
	from matplotlib import rcParams
	#print rcParams
	rcParams.update({'font.size': 14}) # 16
	rcParams.update({'legend.fontsize': 12}) # medium
	rcParams.update({'axes.linewidth': 2.0})
	rcParams.update({'lines.linewidth': 2.0})
	rcParams.update({'patch.linewidth': 2.0})
	rcParams.update({'xtick.major.size': 8.0})
	rcParams.update({'xtick.minor.size': 4.0})
	rcParams.update({'ytick.major.size': 8.0})
	rcParams.update({'ytick.minor.size': 4.0})
	rcParams.update({'xtick.major.width': 1.0})
	rcParams.update({'xtick.minor.width': 1.0})
	rcParams.update({'ytick.major.width': 1.0})
	rcParams.update({'ytick.minor.width': 1.0})
	rcParams.update({'lines.markeredgewidth': 2.0})
	rcParams.update({'axes.formatter.min_exponent': 3})
	#rcParams.update({'hatch.linewidth':  1.0}) # error in pyplot <2.0?

def default_pyplot2_colors(colors):
	colors.ColorConverter.colors['C0']='#1f77b4'
	colors.ColorConverter.colors['C1']='#ff7f0e'
	colors.ColorConverter.colors['C2']='#2ca02c'
	colors.ColorConverter.colors['C3']='#d62728'
	colors.ColorConverter.colors['C4']='#9467bd'
	colors.ColorConverter.colors['C5']='#8c564b'
	colors.ColorConverter.colors['C6']='#e377c2'
	colors.ColorConverter.colors['C7']='#7f7f7f'
	colors.ColorConverter.colors['C8']='#bcbd22'
	colors.ColorConverter.colors['C9']='#17becf'

def save(plt, name, dpi=150):

	# make sure path exists
	ipath= name.rfind('/')
	if ipath!= -1:
		#print name[:ipath]
		if not os.path.isdir(name[:ipath]): os.makedirs(name[:ipath])
	
	plt.savefig(name+'.png',bbox_inches='tight', dpi=dpi)
	plt.close()

def color_array(vals, vmin=None, vmax=None, cmap='jet'):
	# creates colors usable by matplotlib colorbar
	norm = Normalize(vmin=vmin, vmax=vmax)
	# plt.cm.get_cmap
	#Can put any colormap you like here.
	colours = cm.ScalarMappable(norm=norm, cmap=cmap).to_rgba(vals)
	return colours, norm


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as tck
import matplotlib.patches as patches
import helpers
from EPOS import regression

clrs= ['r','g','b','m'] # in epos.prep
fmt_symbol= {'ls':'', 'marker':'o', 'mew':2, 'ms':8,'alpha':0.6}

# backwards compatible colors 
import matplotlib
if matplotlib.__version__[0] != 2: 
	helpers.default_pyplot2_colors(matplotlib.colors)

def periodradius(epos, SNR=True, Model=False, color='C1'):

	f, (ax, axR, axP)= helpers.make_panels(plt, Fancy=True)
	
	sim=epos.synthetic_survey
	title='Simulated Detections: {}'.format(epos.name)
	fsuffix='detect'
	xbins= epos.MC_xvar
	ybins= epos.MC_yvar

	if SNR:
		transit=epos.transit
		title= 'Simulated Transiting Planets'
		fsuffix='transit'
	elif Model:
		pfm= epos.pfm
		fsuffix='population'

		dwR=0.2 # bin width in ln space
		dwP=0.3
		
		xbins= np.exp(np.arange(np.log(epos.mod_xlim[0]),np.log(epos.mod_xlim[-1])+dwP,dwP))
		ybins= np.exp(np.arange(np.log(epos.ytrim[0]),np.log(epos.ytrim[-1])+dwR,dwR))	

	
	''' plot R(P), main panel'''
	f.suptitle(title)
	helpers.set_axes(ax, epos, Trim=True)
	if SNR: 
		ax.plot(transit['P'], transit['Y'], ls='', marker='.', color='C6')
	elif Model:
		ax.plot(pfm['P'],pfm['R'], ls='', marker='.', ms=5.0, color='0.5')
	ax.plot(sim['P'], sim['Y'], ls='', marker='.', color=color)

	if Model:
		xlim= ax.get_xlim()
		ax.set_xlim(xlim[0], epos.mod_xlim[-1])
		#ax.set_ylim(epos.mod_ylim)

	''' Period side panel '''
	#helpers.set_axis_distance(axP, epos, Trim=True)
	axP.set_xlabel(ax.get_xlabel())
	#axP.set_yscale('log')
	#axP.set_ylim([2e-3,5])	
	#axP.set_yticks([0.01,0.1,1])
	#axP.set_yticklabels(['1%','10%','100%'])
	axP.yaxis.tick_right()
	axP.yaxis.set_ticks_position('both')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')
	
	if SNR: 
		axP.hist(transit['P'], bins=xbins, color='C6', density=True)
	elif Model:
		axP.hist(pfm['P'], bins=xbins, color='0.5', density=True, log=True)
		axP.hist(pfm['P'], bins=xbins, color='k', density=True, log=True, 
			histtype='step', zorder=10, ls='-')
	axP.hist(sim['P'], bins=xbins, density=Model, log=Model, color=color)


	''' Radius side panel'''
	#helpers.set_axis_size(axR, epos, Trim=True, In= epos.MassRadius)
	axR.set_ylabel(ax.get_ylabel())

	#axR.set_xscale('log')
	#axR.set_xlim([2e-3,5])
	#axR.set_xticks([1,10,100,1000])
	#axR.set_xticklabels(['1','10','100','1000'], rotation=70)
	for tick in axR.get_xticklabels():
		tick.set_rotation(70)
	#axR.tick_params(axis='x', which='minor',top='off',bottom='off')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')

	if SNR: 
		axR.hist(transit['Y'],orientation='horizontal', bins=ybins, color='C6', 
			density=True)
	elif Model:
		axR.hist(pfm['R'],orientation='horizontal', bins=ybins, color='0.5', 
			density=True, log=True)
		axR.hist(pfm['R'],orientation='horizontal', bins=ybins, color='k', 
			density=True, log=True, histtype='step', zorder=10, ls='-')
	axR.hist(sim['Y'],orientation='horizontal', bins=ybins, 
		density=Model, log=Model, color=color)
	
	# labels
	if SNR:
		xpos= epos.MC_xvar[-1]
		ypos= axP.get_ylim()[-1]/1.05
		axP.text(xpos, ypos, 'Detected ', color=color, ha='right', va='top')
		axP.text(xpos, ypos/1.3, 'Undetected ', color='C6', ha='right', va='top')
	elif Model:
		xpos= ax.get_xlim()[-1]
		ypos= ax.get_ylim()[-1]/1.05
		ax.text(xpos, ypos, 'Detected ', color=color, ha='right', va='top')
		ax.text(xpos, ypos/1.3, 'Undetected ', color='0.5', ha='right', va='top')

	
	#ax.legend(loc='lower left', shadow=False, prop={'size':14}, numpoints=1)
	helpers.save(plt, '{}output/periodradius.{}'.format(epos.plotdir,fsuffix))

def panels(epos, MCMC=False, color='C1'):

	f, (ax, axR, axP)= helpers.make_panels(plt)
	
	sim=epos.synthetic_survey
	
	clr_bf= 'g'
		
	''' plot R(P), main panel'''
	ax.set_title('Simulated Detections')
	helpers.set_axes(ax, epos, Trim=True)
	if epos.MonteCarlo:
		ax.plot(sim['P'], sim['Y'], ls='', marker='.', color=clr_bf if MCMC else 'C0')
	else:
		levels= np.linspace(0,np.max(sim['pdf']))		
		ax.contourf(epos.MC_xvar, epos.MC_yvar, sim['pdf'].T, cmap='Blues', levels=levels)

	''' Period side panel '''
	helpers.set_axis_distance(axP, epos, Trim=True)
	#axP.set_yscale('log')
	#axP.set_ylim([2e-3,5])	
	#axP.set_yticks([0.01,0.1,1])
	#axP.set_yticklabels(['1%','10%','100%'])
	axP.yaxis.tick_right()
	axP.yaxis.set_ticks_position('both')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')
			
	''' Radius side panel'''
	helpers.set_axis_size(axR, epos, Trim=True) #, In= epos.MassRadius)

	#axR.set_xscale('log')
	#axR.set_xlim([2e-3,5])
	#axR.set_xticks([1,10,100,1000])
	#axR.set_xticklabels(['1','10','100','1000'], rotation=70)
	for tick in axR.get_xticklabels():
		tick.set_rotation(70)
	#axR.tick_params(axis='x', which='minor',top='off',bottom='off')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')

	''' Histograms / posterior samples '''
	try:
		xbins= np.geomspace(*epos.xzoom, num=20)
		ybins= np.geomspace(*epos.yzoom, num=10)
	except:
		xbins= np.logspace(*np.log10(epos.xzoom), num=20)
		ybins= np.logspace(*np.log10(epos.yzoom), num=10)
	xscale= np.log(xbins[1]/xbins[0])
	yscale= np.log(ybins[1]/ybins[0])

	if MCMC:
		histkeys= {'color':'b', 'alpha':0.1}
		for ss in epos.ss_sample:
			if epos.MonteCarlo:
				axP.hist(ss['P zoom'], bins=xbins, histtype='step', **histkeys)
				axR.hist(ss['Y zoom'], bins=ybins, orientation='horizontal', \
					histtype='step', **histkeys)
			else:
				axP.plot(ss['P zoom'], ss['P zoom pdf']*xscale, 
					marker='', ls='-', **histkeys)
				axR.plot(ss['Y zoom pdf']*yscale, ss['Y zoom'], 
					marker='', ls='-', **histkeys)
		histdict= {'histtype':'step', 'color':clr_bf}
	else:
		histdict={}
	
	if epos.MonteCarlo:
		axP.hist(sim['P zoom'], bins=xbins, **histdict)
		axR.hist(sim['Y zoom'], bins=ybins, orientation='horizontal', **histdict)
	else:
		axP.plot(sim['P zoom'], sim['P zoom pdf']*xscale, marker='', ls='-')
		axR.plot(sim['Y zoom pdf']*yscale, sim['Y zoom'], marker='', ls='-')

	''' Observations'''
	axP.hist(epos.obs_zoom['x'], bins=xbins,histtype='step', color='C1')
	axR.hist(epos.obs_zoom['y'], bins=ybins, orientation='horizontal',histtype='step', color='C1')
	
	''' Box/ lines'''
	if epos.Zoom:
		for zoom in epos.xzoom: axP.axvline(zoom, ls='--', color='k')
		for zoom in epos.yzoom: axR.axhline(zoom, ls='--', color='k')
		ax.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
			epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )
	
	#ax.legend(loc='lower left', shadow=False, prop={'size':14}, numpoints=1)
	
	fdir= 'mcmc' if MCMC else 'output'
	helpers.save(plt, '{}{}/pdf.zoom'.format(epos.plotdir, fdir))

def pdf_3d(epos):
	#plot in 3D
	# doesn't support log scale on axes, need a workaround
	
	fig = plt.figure()
	ax = fig.gca(projection='3d')

	ax.set_title('Synthetic Model Populations')
	ax.set_xlabel('Orbital Period [days]')
	ax.set_zlabel('Planets/bin')
	
	if epos.RV:	ax.set_ylabel(r'Planet Mass [M$_\bigoplus$]')
	else:		ax.set_ylabel(r'Planet Radius [R$_\bigoplus$]')
	
	ax.set_xlim(np.log10(epos.xtrim))
	ax.set_ylim(np.log10(epos.ytrim))
	xplane, yplane= np.log10(epos.xtrim[0]), np.log10(epos.ytrim[-1])
	ax.set_zlim(0,2000)

	#ax.set_xscale('log')
	#ax.set_yscale('log')

	sim=epos.synthetic_survey
	
	# PDF, individual contributions
	# 	if epos.populationtype is 'model':
	# 		for k, sg in enumerate(epos.groups):
	# 			subset= sim['i sg']==k
	# 			P= sim['P'][subset]
	# 			R= sim['Y'][subset]
	# 			ax.plot(np.log10(P),np.log10(R), zs=0,zdir='z',
	# 				ls='',marker='.',mew=0,ms=5.0,color=clrs[k % 4])
	# 				
	# 			xgrid= np.logspace(*np.log10(epos.xtrim))
	# 			pdf= regression.sliding_window_log(P, None, xgrid) #, width=2. )
	# 			ax.plot(np.log10(xgrid), pdf, zs=yplane,zdir='y', 
	# 				ls='-', marker='', color=clrs[k % 4],
	# 				label='{} x{:.3f}'.format(sg['name'], sg['weight']))
	# 
	# 			ygrid= np.logspace(*np.log10(epos.ytrim))
	# 			pdf= regression.sliding_window_log(R, None, ygrid) #, width=2. )
	# 			ax.plot(np.log10(ygrid), pdf, zs=xplane, zdir='x', 
	# 				ls='-', marker='', color=clrs[k % 4])
	# 	else:

	# top left panel (P,R)
	ax.plot(np.log10(sim['P']), np.log10(sim['Y']),zs=0,zdir='z', 
			ls='', marker='.', mew=0, ms=5.0, color='k')
		
	# PDF, all combined, 2 panels
	xgrid= np.logspace(*np.log10(epos.xtrim))
	pdf= regression.sliding_window_log(sim['P'], None, xgrid) #, width=2. )
	weight= np.sum([sg['weight'] for sg in epos.groups]) if epos.populationtype is 'model' else epos.fitpars.get('pps',Init=True)
	ax.plot(np.log10(xgrid), pdf, zs=yplane,zdir='y', 
		ls='-', marker='', color='k',label='combined x {:.3f}'.format(weight))

	ygrid= np.logspace(*np.log10(epos.ytrim))
	pdf= regression.sliding_window_log(sim['Y'], None, ygrid) #, width=2. )
	ax.plot(np.log10(ygrid), pdf, zs=xplane,zdir='x', ls='-', marker='', color='k')

	# observations
	if epos.Observation and epos.DetectionEfficiency:
		ax.plot(np.log10(epos.obs_xvar), np.log10(epos.obs_yvar), zs=-10,zdir='z',
			ls='', marker='.', mew=0, ms=5.0, color='0.7',zorder=0)
		
		pdf= regression.sliding_window_log(epos.obs_xvar, None, xgrid)
		ax.plot(np.log10(xgrid), pdf, zs=yplane,zdir='y', ls=':', marker='', color='k', label='Kepler')

		pdf= regression.sliding_window_log(epos.obs_yvar, None, ygrid) 
		ax.plot(np.log10(ygrid), pdf, zs=xplane,zdir='x', ls=':', marker='', color='k')
		
	# 	xmax= ax.get_ylim()[-1]
	# 	ymax= ax.get_zlim()[-1]
	# 	ax.errorbar(xmax/1.5, ymax*0.9, yerr=(xmax/1.5*(1.-np.sqrt(1./2.)) ), 
	# 				zs=0,zdir='y', color='k')
	# 	
	# 	xmax= ax.get_xlim()[-1]
	# 	ymax= ax.get_zlim()[-1]
	# 	ax.errorbar(xmax/1.5, ymax*0.9, xerr=(xmax/1.5*(1.-np.sqrt(1./2.)) ), 
	# 				zs=0,zdir='x', color='k')
		
	#ax2.tick_params(labelbottom='on') # does not re-enable axis
	
	''' Legend instead of 4th plot'''
	#ax.legend()
	ax.legend(shadow=False, prop={'size':14},bbox_to_anchor=(0.7,0.0))
	
	ax.view_init(elev=20., azim=-35)
	
	helpers.save(plt, epos.plotdir+'output/pdf.3D.diag')

def pdf(epos):
	#assert epos.populationtype is 'model'
	
	f, ((ax1,ax2),(ax3,ax4)) = plt.subplots(2, 2, sharex='col', sharey='row')
	f.subplots_adjust(hspace=0, wspace=0)
	# 1 2
	# 3 4
	
	ax1.set_title('Synthetic Model Populations',loc='left')

	ax3.set_xlabel('Orbital Period [days]')
	if epos.RV:	ax1.set_ylabel(r'Planet Mass [M$_\bigoplus$]')
	else:		ax1.set_ylabel(r'Planet Radius [R$_\bigoplus$]')
	
	ax2.set_xlabel('Planets/bin')
	ax3.set_ylabel('Planets/bin')
	
	ax1.set_xlim(epos.xtrim)
	ax1.set_ylim(epos.ytrim)

	ax1.set_xscale('log')
	ax1.set_yscale('log')

	sim=epos.synthetic_survey
	
	# PDF, individual contributions
	if epos.populationtype is 'model':
		for k, sg in enumerate(epos.groups):
			subset= sim['i sg']==k
			P= sim['P'][subset]
			R= sim['Y'][subset]
			ax1.plot(P,R, ls='', marker='.', mew=0, ms=5.0, color=clrs[k % 4])
				
			xgrid= np.logspace(*np.log10(epos.xtrim))
			pdf= regression.sliding_window_log(P, None, xgrid) #, width=2. )
			ax3.plot(xgrid, pdf, ls='-', marker='', color=clrs[k % 4],label='{} x{:.3f}'.format(sg['name'], sg['weight']))

			ygrid= np.logspace(*np.log10(epos.ytrim))
			pdf= regression.sliding_window_log(R, None, ygrid) #, width=2. )
			ax2.plot(pdf, ygrid, ls='-', marker='', color=clrs[k % 4])
	else:
		# top left panel (P,R)
		ax1.plot(sim['P'], sim['Y'], ls='', marker='.', mew=0, ms=5.0, color='k')
		
	# PDF, all combined, 2 panels
	xgrid= np.logspace(*np.log10(epos.xtrim))
	pdf= regression.sliding_window_log(sim['P'], None, xgrid) #, width=2. )
	weight= np.sum([sg['weight'] for sg in epos.groups]) if epos.populationtype is 'model' else epos.fitpars.get('pps',Init=True)
	ax3.plot(xgrid, pdf, ls='-', marker='', color='k',label='combined x {:.3f}'.format(weight))

	ygrid= np.logspace(*np.log10(epos.ytrim))
	pdf= regression.sliding_window_log(sim['Y'], None, ygrid) #, width=2. )
	ax2.plot(pdf, ygrid, ls='-', marker='', color='k')

	# observations
	if epos.Observation and epos.DetectionEfficiency:
		pdf= regression.sliding_window_log(epos.obs_xvar, None, xgrid)
		ax3.plot(xgrid, pdf, ls=':', marker='', color='k', label='Kepler')

		pdf= regression.sliding_window_log(epos.obs_yvar, None, ygrid) 
		ax2.plot(pdf, ygrid, ls=':', marker='', color='k')
	
	xmax= ax3.get_xlim()[-1]
	ymax= ax3.get_ylim()[-1]
	ax3.errorbar(xmax/1.5, ymax*0.9, xerr=(xmax/1.5*(1.-np.sqrt(1./2.)) ), color='k')
	
	xmax= ax2.get_xlim()[-1]
	ymin= ax2.get_ylim()[0]
	ax2.errorbar(xmax*0.9, ymin*1.5, yerr=(ymin*1.5*(1.-np.sqrt(1./2.)) ), color='k')
	
	#ax2.tick_params(labelbottom='on') # does not re-enable axis
	
	''' Legend instead of 4th plot'''
	ax3.legend(loc='center left', shadow=False, prop={'size':14}, numpoints=1,bbox_to_anchor=(1,0.5))
	ax4.axis('off')
	
	helpers.save(plt, epos.plotdir+'output/pdf.diag')

def cdf(epos, color='C1'):
	
	f, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2)
	f.set_size_inches(9, 7) # default 7, 5

	f.suptitle(epos.name)

	''' 
	top left: synthetic obsservation
	'''
	sim= epos.synthetic_survey
	helpers.set_axes(ax1, epos, Trim=True)
	ax1.set_title('Synthetic ({})'.format(sim['nobs']))
	if epos.MonteCarlo:
		ax1.plot(sim['P'], sim['Y'], ls='', marker='.', mew=0, ms=5.0, color=color, alpha=0.5)
	else:
		levels= np.linspace(0,np.max(sim['pdf']))		
		ax1.contourf(epos.MC_xvar, epos.MC_yvar, sim['pdf'].T, cmap='Reds', levels=levels)
	
	if epos.Zoom:
		ax1.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
			epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )
		
	''' 
	Top right: observed population 
	'''
	ax2.set_title('Observed ({})'.format(epos.obs_zoom['x'].size))
	helpers.set_axes(ax2, epos, Trim=True)

	ax2.plot(epos.obs_xvar, epos.obs_yvar, ls='', marker='.', mew=0, ms=5.0, color='C3')		
	if epos.Zoom:
		ax2.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
			epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )

	''' 
	cdf orbital period 
	'''
	if 'xvar' in epos.prob:
		ax3.set_title('Period, p={:.3g}'.format(epos.prob['xvar']))
	else:
		ax3.set_title('Orbital Period')
	ax3.set_xlabel('Orbital Period [days]')
	ax3.set_ylabel('CDF')
	ax3.set_xscale('log')
	ax3.set_ylim([-0.05,1.05])

	ax3.set_xlim(*epos.xzoom)

	if epos.MonteCarlo:
		#model histogram x
		P= sim['P zoom']
		ax3.plot(np.sort(P), np.arange(P.size, dtype=float)/P.size, ls='-', marker='', color=color)
	else:
		ax3.plot(sim['P zoom'], sim['P zoom cdf'], ls='-', marker='', color='r')
	
	#obs histogram x
	P= epos.obs_zoom['x']
	ax3.plot(np.sort(P), np.arange(P.size, dtype=float)/P.size, ls='-', marker='', color='C3')		

	''' 
	CDF planet radius
	'''
	if 'yvar' in epos.prob:
		ax4.set_title('{}, p={:.3g}'.format('M sin i' if epos.RV else 'Radius', epos.prob['yvar']))
	else:
		ax4.set_title('{}'.format('M sin i' if epos.RV else 'Radius'))
	if epos.RV:	ax4.set_xlabel(r'Planet M sin i [M$_\bigoplus$]')
	else:		ax4.set_xlabel(r'Planet Radius [R$_\bigoplus$]')
	ax4.set_ylabel('CDF')
	ax4.set_xscale('log')
	ax4.set_ylim([-0.05,1.05])
	ax4.set_xlim(*epos.yzoom)
	
	ax4.set_xticks(epos.yticks)
	ax4.get_xaxis().set_major_formatter(tck.ScalarFormatter())

	if epos.MonteCarlo:
		#model histogram x
		R= sim['Y zoom']
		ax4.plot(np.sort(R), np.arange(R.size, dtype=float)/R.size, ls='-', marker='', color=color)
	else:
		ax4.plot(sim['Y zoom'], sim['Y zoom cdf'], ls='-', marker='', color=color)

	#obs histogram x
	R= epos.obs_zoom['y']
	ax4.plot(np.sort(R), np.arange(R.size, dtype=float)/R.size, ls='-', marker='', color='C3')		
		
	f.tight_layout()
	helpers.save(plt, epos.plotdir+'output/cdf.diag')


"""
EPOS classes docstring

This module defines the EPOS class, that contains the observed exoplanets,
the survey detection efficiency, and the synthetic planet population.
see example.py for a simple demonstration of the class
"""

import numpy as np

import cgs
import EPOS.multi
from EPOS.plot.helpers import set_pyplot_defaults
from EPOS import __version__

class fitparameters:
	''' Holds the fit parameters. Usually initialized in epos.fitpars '''
	def __init__(self):
		self.fitpars={} # not in order
		self.keysall=[]
		self.keysfit=[]
		self.keys2d=[]
		self.keypps='pps'
	
	def add(self, key, value, fixed=False, min=-np.inf, max=np.inf, 
				dx=None, text=None, is2D=False, isnorm=False):
		'''Add a fit parameter
		
		Args:
			key(str): fit parameter dictionary key
			value(float): starting guess
			fixed(bool): keep this parameter fixed
			min(float): lower bound
			max(float): upper bound
			dx(float): initial dispersion for MCMC
			text(str): plot safe name?
			is2D(bool): use this parameter in the 2D parametric :meth:`EPOS.fitfunctions`
			isnorm(bool): this parameter is the normalization factor for the number of planet per star :meth:`EPOS.fitfunctions`
		'''
		fp=self.fitpars[key]= {}
		
		# list of keys
		self.keysall.append(key)
		if is2D: self.keys2d.append(key)
		if isnorm: self.keypps=key
		if not fixed: self.keysfit.append(key)
		
		fp['key']= key
		fp['value_init']= value

		# T/F
		fp['fixed']=fixed
		
		# parameters for fitting
		if not fixed:
			fp['min']=min
			fp['max']=max
			# initial walker positions, can't be zero, default 10% or 0.1 dex
			dx=0.1*value if dx is None else dx
			fp['dx']=abs(dx) if (dx!=0) else 0.1

	def default(self, key, value, Verbose=True):
		if not key in self.keysall: 
			if Verbose: print '  Set {} to default {}'.format(key, value)
			self.add(key, value, fixed=True)
	
	def set(self, key, value):
		self.fitpars[key]['value_init']=value

	def setfit(self, mclist):
		for i,key in enumerate(self.keysfit):
			#self.fitpars[key]['value_fit']=mclist[self.keysfit.index(key)]
			self.fitpars[key]['value_fit']=mclist[i]
	
	def get(self, key, Init=False, attr=None):
		if attr is None:
			# return initial or fit value
			if Init or not 'value_fit' in self.fitpars[key]:
				return self.fitpars[key]['value_init']
			else:
				return self.fitpars[key]['value_fit']
		else:
			# list of attribute
			return self.fitpars[key][attr]
	
	def get2d(self, Init=False):
		# returns the values for the 2D distribution
		return [self.get(key, Init=Init) for key in self.keys2d]

	def getpps(self, Init=False):
		# returns the normalization for the 2D distribution
		return self.get(self.keypps, Init=Init)

	def getpps_fromlist(self, parlist):
		return self.getmc(self.keypps, parlist)
	
	def getfit(self, Init=True, attr=None): 
		return [self.get(key, Init=Init, attr=attr) for key in self.keysfit]

	def getmc(self, key, parlist):
				
		# returns value for an mc run
		if self.fitpars[key]['fixed']:
			return self.fitpars[key]['value_init']
		else:
			return parlist[self.keysfit.index(key)]
			#try:				
			#except ValueError:
			#	raise ValueError('Parameter {} not defined'.format(key))

	def get2d_fromlist(self, parlist):
		# returns 2d from fit/fixed, fit supplied in list
		l2d= []
		for key in self.keys2d:
			if self.fitpars[key]['fixed']:
				l2d.append(self.fitpars[key]['value_init'])
				#pps= self.fitpars['pps']['value_init']
			else:
				l2d.append(parlist[self.keysfit.index(key)])
				#pps= parlist[self.keysfit.index('pps')]
		return l2d
	
	def checkbounds(self, parlist):
		for i, key in enumerate(self.keysfit):
			if parlist[i]<self.fitpars[key]['min']:
				raise ValueError('{} out of bounds, {} < {}'.format(
					key,parlist[i],self.fitpars[key]['min']))
			if parlist[i]>self.fitpars[key]['max']:
				raise ValueError('{} out of bounds, {} > {}'.format(
					key,parlist[i],self.fitpars[key]['max']))

	def get_kfree(self):
		return len(self.keysfit)

class epos:
	"""The epos class
	
	Description:
		Initialize

	Args:
		name (str): name to use for directories
		RV(bool): Compare to radial velocity instead of transits
		MC(bool): Generate planet population by random draws 
		Msini(bool): Convert the planet mass distribution into an Msini distribution
		Debug(bool): Log more output for debugging
		seed(int): Same random number for each simulation? True, None, or int
		Norm(bool): normalize pdf (deprecated?)
	
	Attributes:
		name(str): name
		plotdir(str): plot directory
		RV(bool): Compare to Radial Velocity instead of transit data
		Multi(bool): Do multi-planet statistics
		RandomPairing(bool): multis are randomly paired
		Isotropic(bool): Assume isotropic mutual inclinations
		Parametric(bool): parametric planet population?
		Debug(bool): Verbose logging
		seed(): Random seed, can be any of int, True, or None
	"""
	def __init__(self, name, Debug=False, seed=True, title=None, 
		RV=False, Norm=False, MC=True, Msini=False):
		"""
		Initialize the class
		"""
		self.name=name
		self.title=name if title is None else title

		print '\n\n |~| epos {} |~|\n'.format(__version__)

		''' Directories '''
		self.plotdir='png/{}/'.format(name)
		self.jsondir='json/{}/'.format(name)
		#self.path= os.path.dirname(EPOS.__file__)

		''' EPOS mode'''
		self.RV= RV
		self.Msini= Msini # do an M -> Msini conversion
		self.Multi=False
		self.RandomPairing= False
		self.Isotropic= False # phase out?
		self.MonteCarlo= MC
		
		# Seed for the random number generator
		if seed is None: self.seed= None
		else:
			if type(seed) is int: self.seed= seed
			else: self.seed= np.random.randint(0, 4294967295)
			print '\nUsing random seed {}'.format(self.seed)
		
		self.Debug= False
		self.Parallel= True # speed up a few calculations 
		set_pyplot_defaults() # nicer plots

		# switches to be set later (undocumented)	
		self.Observation=False
		self.Range=False
		self.DetectionEfficiency=False
		self.Occurrence= False # inverse detection efficiency (?)
		self.Prep= False # ready to run? EPOS.run.once()
		self.MassRadius= False
		self.Radius= False # is this used?
		self.PDF=False
		
		self.plotpars={} # dictionary to hold some customization keywords

	def set_observation(self, xvar, yvar, starID, nstars=1.6862e5, 
		radiusError=0.1, score=None):
		''' Observed planet population
		
		Args:
			xvar: planet orbital period [list]
			yvar: planet radius or M sin i [list]
			ID: planet ID [list]
			nstars: number of stars surveyed
		
		Note:
			Some pre-defined planet populations from Kepler can be generated from 
			:mod:`EPOS.kepler.dr25()`
		'''
		order= np.lexsort((xvar,starID)) # sort by ID, then P
		self.obs_xvar=np.asarray(xvar)[order]
		self.obs_yvar=np.asarray(yvar)[order]
		self.obs_starID=np.asarray(starID)[order]
		self.nstars=nstars

		if score is not None:
			self.obs_score= np.asarray(score)[order]
		
		assert self.obs_xvar.ndim == self.obs_yvar.ndim == self.obs_starID.ndim == 1, 'only 1D arrays'
		assert self.obs_xvar.size == self.obs_yvar.size == self.obs_starID.size, 'arrays not same length'
	
		# set plot limits in observation 5% wider than data
		xmin,xmax= min(self.obs_xvar), max(self.obs_xvar)
		ymin,ymax= min(self.obs_yvar), max(self.obs_yvar)
		dx= (xmax/xmin)**0.05
		dy= (ymax/ymin)**0.05
		self.obs_xlim=[xmin/dx,xmax*dx]
		self.obs_ylim=[ymin/dy,ymax*dy]
		
		self.Observation=True
		self.radiusError= radiusError
		
		# print some stuff
		print '\nObservations:\n  {} stars'.format(int(nstars))
		print '  {} planets'.format(self.obs_starID.size)
		EPOS.multi.indices(self.obs_starID, Verbose=True)
		epos.multi={}
		epos.multi['bin'], epos.multi['count']= \
			EPOS.multi.frequency(self.obs_starID, Verbose=True)
		epos.multi['pl cnt']= epos.multi['bin']* epos.multi['count']
		epos.multi['Pratio'], epos.multi['Pinner'], epos.multi['Rpair']= \
			EPOS.multi.periodratio(self.obs_starID, self.obs_xvar, R=self.obs_yvar, Verbose=True)
		epos.multi['cdf']= EPOS.multi.cdf(self.obs_starID, Verbose=True)	
	
	def set_survey(self, xvar, yvar, eff_2D, Rstar=1.0, Mstar=1.0, vet_2D=None):
		'''Survey detection efficiency (completeness)
		Args:
			xvar: planet orbital period grid [list]'
			yvar: planet radius or M sin i grid [list]
			eff_2D: 2D matrix of detection efficiency
			Rstar: stellar radius, for calculating transit probability
			Mstar: stellar mass, for period-semimajor axis conversion
		
		Note:
			Some pre-defined detection efficiencies from Kepler can be generated from 
			:mod:`EPOS.kepler`
		'''
		self.eff_xvar=np.asarray(xvar)
		self.eff_yvar=np.asarray(yvar)
		self.eff_2D=np.asarray(eff_2D)
		
		assert self.eff_xvar.ndim == self.eff_yvar.ndim == 1, 'only 1D arrays'
		assert self.eff_2D.ndim == 2, 'Detection efficiency must by a 2dim array'
		if self.eff_2D.shape != (self.eff_xvar.size, self.eff_yvar.size):
			raise ValueError('Mismatching detection efficiency'+
			'\n: nx={}, ny={}, (nx,ny)=({},{})'.format(self.eff_xvar.size, 
				self.eff_yvar.size, *self.eff_2D.shape))
	
		self.eff_xlim= [min(self.eff_xvar),max(self.eff_xvar)]
		self.eff_ylim= [min(self.eff_yvar),max(self.eff_yvar)]
		
		self.Mstar= Mstar
		if self.RV:
			self.completeness= self.eff_2D
		else:
			self.Rstar=Rstar # Solar radii
			self.Pindex= -2./3.
			fourpi2_GM= 4.*np.pi**2. / (cgs.G*self.Mstar*cgs.Msun)
			self.fgeo_prefac= self.Rstar*cgs.Rsun * fourpi2_GM**(1./3.) / cgs.day**(2./3.)
			#print self.fgeo_prefac
			P, R= np.meshgrid(self.eff_xvar, self.eff_yvar, indexing='ij')
			
			self.completeness= self.eff_2D * self.fgeo_prefac*P**self.Pindex
		
		if vet_2D is not None:
			self.vetting= np.asarray(vet_2D)
			
			if self.eff_2D.shape != (self.eff_xvar.size, self.eff_yvar.size):
				raise ValueError('Mismatching vetting efficiency'+
					'\n: (nx,ny)={}, (nx,ny={})'.format(self.eff_2D.shape,
								self.vetting.shape))

			self.completeness_novet= self.completeness
			self.completeness*= self.vetting

		self.DetectionEfficiency=True
	
	def set_ranges(self, xtrim=None, ytrim=None, xzoom=None, yzoom=None, 
		LogArea=False, Occ=False, UnitTicks=True, plotxgrid= None, plotygrid=None):
		
		if self.Range: raise ValueError('Range already defined')
		if not self.Observation: raise ValueError('No observation defined')
		if not self.DetectionEfficiency: raise ValueError('No detection effifiency defined')
		
		''' Define the region where completeness is calculated'''
		if xtrim is None:
			print 'Trimming x-axis from detection efficiency'
			self.xtrim= self.eff_xlim
		else:
			self.xtrim= [max(xtrim[0], self.eff_xlim[0]), min(xtrim[1], self.eff_xlim[1])]

		if ytrim is None:
			print 'Trimming y-axis from detection efficiency'
			self.ytrim= self.eff_ylim
		else:
			self.ytrim= [max(ytrim[0], self.eff_ylim[0]), min(ytrim[1], self.eff_ylim[1])]
		
		''' Define a smaller region where observational comparison is performed'''	
		if xzoom is None:
			print 'Not zooming in on x-axis for model comparison'
			self.xzoom= self.xtrim
		else:
			self.xzoom= [max(xzoom[0], self.xtrim[0]), min(xzoom[1], self.xtrim[1])]

		if yzoom is None:
			print 'Not zooming in on y-axis for model comparison'
			self.yzoom= self.ytrim
		else:
			self.yzoom= [max(yzoom[0], self.ytrim[0]), min(yzoom[1], self.ytrim[1])]
		
		if (xzoom is None) and (yzoom is None):
			self.Zoom=False
		elif (self.xzoom==self.xtrim) and (self.yzoom==self.ytrim):
			self.Zoom=False
			print 'Trim equal to zoom' # so??
		else:
			self.Zoom=True
		
		''' 
		Prep the Monte Carlo grid for the observable
		'''
		# make sure range _encompasses_ trim
		ixmin,ixmax= _trimarray(self.eff_xvar, self.xtrim)
		iymin,iymax= _trimarray(self.eff_yvar, self.ytrim)
		self.trim_to_zoom= (slice(ixmin,ixmax),slice(iymin,iymax))
	
		self.MC_xvar= self.eff_xvar[ixmin:ixmax]
		self.MC_yvar= self.eff_yvar[iymin:iymax]
		self.MC_eff= self.eff_2D[ixmin:ixmax,iymin:iymax]
		if hasattr(self,'vetting'):
			self.MC_eff*= self.vetting[ixmin:ixmax,iymin:iymax]
		
		# scale factor to multiply pdf such that occurrence in units of dlnR dlnP
		if LogArea:
			area= np.log10
			self.plotpars['area']= 'dlog'
		else:
			area= np.log
			self.plotpars['area']= 'dln'

		self.scale_x= self.MC_xvar.size/area(self.MC_xvar[-1]/self.MC_xvar[0])
		self.scale_y= self.MC_yvar.size/area(self.MC_yvar[-1]/self.MC_yvar[0])
		self.scale= self.scale_x * self.scale_y
		
		self.X, self.Y= np.meshgrid(self.MC_xvar, self.MC_yvar, indexing='ij')
		
		''' Transit probability '''
		if not self.RV:
			self.f_geo= self.fgeo_prefac *self.X**self.Pindex
			self.f_det= self.f_geo * self.MC_eff
		
		'''
		Prep the grid for the non-MC simulation
		'''
		if not self.MonteCarlo:
			self.noMC_zoom_x=np.geomspace(self.xzoom[0],self.xzoom[-1])
			self.noMC_zoom_y=np.geomspace(self.yzoom[0],self.yzoom[-1])
			self.noMC_scale_x= self.noMC_zoom_x.size/ np.log(self.xzoom[-1]/self.xzoom[0])
			self.noMC_scale_y= self.noMC_zoom_y.size/ np.log(self.yzoom[-1]/self.yzoom[0])
			self.noMC_scale= self.noMC_scale_x * self.noMC_scale_y
		
		''' Prep the grid for the PDF, if using a mass-radius conversion'''
		if self.PDF:
			if self.MassRadius:
				self.in_ytrim= self.masslimits
				self.in_yvar= np.logspace(*np.log10(self.in_ytrim))
				self.scale_in_y= \
					self.in_yvar.size/area(self.in_yvar[-1]/self.in_yvar[0])	
				self.scale_in= self.scale_x * self.scale_in_y
				self.X_in,self.Y_in= np.meshgrid(self.MC_xvar,self.in_yvar,indexing='ij')
			elif self.RV and self.Msini:
				# extend grid to higher masses
				nex=13
				ext= self.MC_yvar[-nex:]*self.MC_yvar[-1]/self.MC_yvar[-(nex+1)]
				self.in_yvar= np.concatenate([self.MC_yvar, ext]) 
				self.in_ytrim= self.in_yvar[([0,-1])]
				self.scale_in_y= self.scale_y 
				self.scale_in= self.scale
				self.X_in, self.Y_in= np.meshgrid(self.MC_xvar,self.in_yvar,indexing='ij')	
			else:
				self.in_ytrim= self.ytrim
				self.in_yvar= self.MC_yvar
				self.scale_in_y= self.scale_y 
				self.scale_in= self.scale
				self.X_in, self.Y_in= self.X, self.Y
			
		''' plot ticks '''
		yr= 365.24
		if UnitTicks:
			Pticks=np.array([1,10,100, yr, 10.*yr])
			Pticklabels= np.array(['1','10','100','1yr','10yr'])
		else:
			Pticks= np.logspace(-1,6,8)
			Pticklabels= Pticks

		xrange= (self.xtrim[0]<=Pticks) & (Pticks<=self.xtrim[1])
		self.xticks= Pticks[xrange]
		self.xticklabels= Pticklabels[xrange]
		
		if UnitTicks:
			Mjup= cgs.Mjup/cgs.Mearth
			Mticks= np.array([0.1,1,10,100, Mjup, 10.*Mjup, 100.*Mjup])
			Mticklabels= np.array(['0.1','1','10','100', '$M_J$', '$10 M_J$', '$100 M_J$'])
			Rticks= np.array([0.25,0.5,1,2, 4, 10])
			Rticklabels= np.array(['0.25','0.5','1','2', '4','10'])
		else:
			Mticks= np.logspace(-1,6,8)
			Mticklabels= Mticks
			Rticks= np.logspace(-2,4,7)
			Rticklabels= Rticks

		if self.MassRadius:
			yinrange= (self.in_ytrim[0]<=Mticks) & (Mticks<=self.in_ytrim[1])
			self.y_inticks= Mticks[yinrange]
			self.y_inticklabels= Mticklabels[yinrange]
			yrange= (self.ytrim[0]<=Rticks) & (Rticks<=self.ytrim[1])
			self.yticks= Rticks[yrange]
			self.yticklabels= Rticklabels[yrange]			
		elif self.RV:
			yrange= (self.ytrim[0]<=Mticks) & (Mticks<=self.ytrim[1])
			self.yticks= Mticks[yrange]
			self.yticklabels= Mticklabels[yrange]
			yrange= (self.in_ytrim[0]<=Mticks) & (Mticks<=self.in_ytrim[1])
			self.y_inticks= Mticks[yrange]
			self.y_inticklabels= Mticklabels[yrange]
		else:			
			yrange= (self.ytrim[0]<=Rticks) & (Rticks<=self.ytrim[1])
			self.yticks= Rticks[yrange]
			self.yticklabels= Rticklabels[yrange]
			
		self.Range=True
		
		if Occ:
			if self.MassRadius:
				raise ValueError('Plotting occurrence with mass-radius not yet supported')
				#pass
				
			if not hasattr(self,'occurrence'):
				self.occurrence={}
			focc= self.occurrence
			
			if self.RV:
				xgrid= np.geomspace(self.MC_xvar[0], self.MC_xvar[-1], num=20)
				ygrid= np.geomspace(self.MC_yvar[0], self.MC_yvar[-1], num=20)
			else:
				xgrid= self.MC_xvar
				ygrid= self.MC_yvar	
			
			# custom grid for *plotting* occurrence rates
			if plotxgrid is not None: xgrid= plotxgrid
			if plotygrid is not None: ygrid= plotygrid

			focc['xzoom']={}
			#focc['xzoom']['grid']= xgrid
			#ygrid= np.exp(np.arange(np.log(self.MC_yvar[0]),np.log(self.MC_yvar[-1])+0))
			focc['xzoom']['x']= [self.xzoom]* (ygrid.size-1)
			focc['xzoom']['y']= [[i,j] for i,j in zip(ygrid[:-1],ygrid[1:])]

			focc['yzoom']={}
			#focc['yzoom']['grid']= ygrid
			#xgrid= np.exp(np.arange(np.log(self.MC_xvar[0]),np.log(self.MC_xvar[-1])+0))
			focc['yzoom']['x']= [[i,j] for i,j in zip(xgrid[:-1],xgrid[1:])]
			focc['yzoom']['y']= [self.yzoom]* (xgrid.size-1)

		#if hasattr(self, pfm):
	
	def set_bins(self, xbins=[[1,10]], ybins=[[1,10]],xgrid=None, ygrid=None,Grid=False):
		'''
		Initialize period-radius (or mass) bins for occurrence rate calculations
		
		Description:
			Bins can be generated from a grid, f.e. xgrid=[1,10,100], 
			or from a list of bin edges, f.e. xbins= [[1,10],[10,100]]
		
		Args:
			xbins(list):	(list of) period bin edges
			ybins(list):	(list of) radius/mass bin edges
			xgrid(list):	period bin in interfaces
			ygrid(list):	radius/mas bin interfaces
			Grid(bool):
				If true, create a 2D grid from bins: nbins = nx ``*`` ny.
				If false, pair x and y bins: nbins == nx == ny
		'''
		if not hasattr(self,'occurrence'):
			self.occurrence={}
		focc= self.occurrence
		
		#generate list of bin inner and outer edges
		if xgrid is None:
			if np.ndim(xbins) ==1:
				assert len(xbins)==2
				_xbins= [xbins]
			elif np.ndim(xbins) ==2: 
				_xbins= xbins
			else:
				raise ValueError('wrong bin dimensions')
		else:
			_xbins= np.array([[i,j] for i,j in zip(xgrid[:-1],xgrid[1:])])

		if ygrid is None:
			if np.ndim(ybins) ==1:
				assert len(ybins)==2
				_ybins= [ybins]
			elif np.ndim(ybins) ==2: 
				_ybins= ybins
			else:
				raise ValueError('wrong bin dimensions')
		else:
			_ybins= np.array([[i,j] for i,j in zip(ygrid[:-1],ygrid[1:])])
		
		focc['bin']={}
		if Grid:
			focc['bin']['x']= np.tile(_xbins, (len(_ybins),1))
			#focc['bin']['y in']= np.tile(_ybins, (len(_xbins),1))
			focc['bin']['y in']= np.repeat(_ybins, len(_xbins), axis=0)
			# TODO: store 1d -> 2d mapping
			
		else:		
			if np.shape(_xbins) != np.shape(_ybins):
				raise ValueError('unequal amount of bins. Use Grid=True?')
			focc['bin']['x']= _xbins
			focc['bin']['y in']= _ybins

	def set_bins_poly(self, polys, labels=None):
		'''
		Initialize polygonic bins for occurrence rate calculations
		
		Description:
			each polygone is a Nx2 numpy array of (x,y) coordinates
		
		Args:
			polys(list):	(list of) polygones
		'''
		if not hasattr(self,'occurrence'):
			self.occurrence={}
		focc= self.occurrence
		
		focc['poly']={}
		focc['poly']['coords']=[]

		print '\nTrying {} polygons'.format(len(polys))
		for coords in polys:
			npc= np.asarray(coords)
			assert npc.ndim==2, 'coords needs to de 2dim list'
			assert npc.shape[-1]==2, 'coords need to be 2xN array'
			focc['poly']['coords'].append(npc)
	
		if labels is not None:
			assert len(labels) == len(focc['poly']['coords']), 'Names don\'t match up'
			focc['poly']['labels']= np.asarray(labels)

	def set_parametric(self, func):
		'''Define a parametric function to generate the planet size-period distribution
		
		Description:
			Function should be callable as func(X, Y, \*fitpars2d) with
			X(np.array): period
			Y(np.array): size (radius or mass)
			The list of fit parameters fitpars2d will be constructed from parameters 
			added using :func:`EPOS.fitparameters.add` with is2D=True
			
		Note:
			Some pre-defined functions can be found in :mod:`EPOS.fitfunctions`
		
		Args:
			func (function): callable function
		
		'''		
		if not callable(func): raise ValueError('func is not a callable function')		
		self.func=func
		
		self.pdfpars= fitparameters()
		
		self.Parametric= True
		self.PDF=True	
		self.fitpars=self.pdfpars
		self.summarystatistic= ['N','xvar','yvar']
	
	def set_multi(self, spacing=None):
		if not self.Parametric:
			raise ValueError('Define a parametric planet population first')
		self.Multi=True
		
		self.RandomPairing= (spacing==None)
		self.spacing= spacing # None, brokenpowerlaw, dimensionless

		# skipping yvar here
		self.summarystatistic= ['N','xvar','Nk','dP','Pin']
	
	def set_population(self, name, sma, mass, 
		radius=None, inc=None, starID=None, ecc=None, tag=None, 
		Verbose=False, **kwargs):
		# tag is fit parameter, i.e. metallicity, surface density, or model #

		if Verbose:
			print '\nArguments not used by set_population:'
			print '  ',kwargs.keys()

		if hasattr(self, 'pfm'):
			raise ValueError('expand: adding multiple populations?')
		
		self.Parametric=False
		self.modelpars= fitparameters()
		self.fitpars= self.modelpars
		
		if inc is None:
			self.summarystatistic= ['N'] # xvar, yvar?
		else:
			self.summarystatistic= ['N','Nk','dP'] #,'Pin']
		
		# length checks
		try:
			if len(sma) != len(mass): 
				raise ValueError('sma ({}) and mass ({}) not same length'.format(len(sma),len(mass)))
		except: 
			raise ValueError('sma ({}) and mass ({}) have to be iterable'.format(type(sma), type(mass)))
		
		# model has mutual inclinations?
		self.Multi= (inc != None)
		
		pfm= self.pfm= {}
		pfm['name']= name
		
		# lexsort?
		pfm['sma']= np.asarray(sma)
		pfm['M']= np.asarray(mass)
		
		# ID is array 0...ns with np elements
		if starID is None:
			pfm['ID']= np.arange(len(sma))
		else:
			_, pfm['ID']= np.unique(starID, return_inverse=True)
			
		if radius is not None:
			pfm['R']= np.asarray(radius)
		if ecc is not None:
			pfm['ecc']= np.asarray(ecc)			
		if tag is not None:
			pfm['tag']= np.asarray(tag)		
		if inc is None:
			self.Multi=False
		else:
			self.Multi= True
			pfm['inc']= np.asarray(inc)
				
		pfm['P']= pfm['sma']**1.5 * 365.25 # update
		
		pfm['np']= pfm['ID'].size
		pfm['ns']= np.unique(pfm['ID']).size
		
		''' If multiple planets per stars: Lexsort, period ratio'''
		if pfm['np'] > pfm['ns']:
			
			order= np.lexsort((pfm['sma'],pfm['ID'])) # sort by ID, then sma
			for key in ['ID','sma','M','P','inc','tag']:
				if key in pfm: pfm[key]=pfm[key][order]
			
			EPOS.multi.indices(pfm['ID'], Verbose=True)
			pfm['k'], pfm['Nk']= EPOS.multi.frequency(pfm['ID'], Verbose=True)
			
			# period ratio, multi-planet index
			single, multi, ksys, multis= EPOS.multi.nth_planet(pfm['ID'],pfm['P'])
			pfm['dP']=np.ones_like(pfm['P'])
			pfm['kth']=np.zeros_like(pfm['P'], dtype=int)

			pfm['dP'][single]= 0 #np.nan
			for k, km in enumerate(multis[1:]):
				# 2nd, 3rd, 4th??
				pfm['dP'][km]= pfm['P'][km]/pfm['P'][np.array(km)-1]
				pfm['kth'][km]= k+1
			#print pfm['ID'][1:6]
			#print pfm['P'][1:6]
			#print pfm['dP'] # not ok?	
			#for a,b in zip(pfm['ID'], pfm['kth']):
			#	print a,b

			# innermost planet in multi
			pfm['Pin']= np.squeeze(pfm['P'][np.array(multis[1])-1])

			pfm['system index']= np.arange(pfm['ns']) # 0...ns with ns elements
			pfm['planet index']= np.arange(pfm['np']) # 0...np with np elements

			if 'tag' in pfm:
				_, index= np.unique(starID, return_index=True)
				pfm['system tag']= pfm['tag'][index]
			#pfm['tag'] == pfm['system tag'][pfm['ID']]
		else:
			pfm['system tag']= pfm['tag']

		pfm['M limits']=[np.min(pfm['M']),np.max(pfm['M'])]
		pfm['P limits']=[np.min(pfm['P']),np.max(pfm['P'])]
		
		# set plot limits in model 5% wider than data
		xmin,xmax= min(pfm['P']), max(pfm['P'])
		ymin,ymax= min(pfm['M']), max(pfm['M'])
		dx= (xmax/xmin)**0.05
		dy= (ymax/ymin)**0.05
		self.mod_xlim=[xmin/dx, xmax*dx]
		self.mod_ylim=[ymin/dy, ymax*dy]
		# ??
		self.mod_xlim=[min(xmin/dx, self.mod_xlim[0]),max(xmax*dx, self.mod_xlim[1])]
		self.mod_ylim=[min(ymin/dy, self.mod_ylim[0]),max(ymax*dy, self.mod_ylim[1])]
	
	def set_massradius(self, MR, name, masslimits= [0.01,1e3]):
		print '\nMass-Radius relation from {}'.format(name)
		if self.MassRadius:
			raise ValueError('Already defined a Mass-Radius conversion function ')
		# actually radius as funtion of mass (mass-to-radius)
		if not callable(MR): raise ValueError('Mass-Radius function is not callable')

		self.MassRadius=True
		self.MR=MR
		self.MR_label=name
		
		self.masslimits=masslimits 
		meanradius= MR(masslimits)[0]
		print 'Mass and Radius limits:'
		print '  min M = {:.3f}-> <R> ={:.2f}'.format(masslimits[0], meanradius[0] )
		print '  max M = {:.0f}-> <R> ={:.1f}'.format(masslimits[-1], meanradius[-1] )

def _trimarray(array,trim):
	# trims array of points not needed for interpolation
	if trim[0] < array[1]:
		imin=0
	else:
		imin = np.searchsorted(array, trim[0], side='left')-1
	
	if trim[1] > array[-2]:
		imax=len(array)
	else:
		imax = np.searchsorted(array, trim[1], side='left')+1
	
	return imin, imax



#! /usr/bin/env ipython
''' 
This script is meant to reproduce some of the results/figures from
Fernandes et al. 2019 (AJ submitted), in particular the 3 and 4 parameter 
solutions in Table 1 and 2 and Figure 4, (9), and 10

Text output will appear in the terminal.

Symmetric:
./paper_fernandes_2019_broken_powerlaw.py sym

Best-fit values
  pps= 0.476 +0.0798 -0.0704
  P break= 1.54e+03 +948 -381
  p1= 0.652 +0.197 -0.17
  m1= -0.442 +0.0526 -0.0544

  posterior per bin
  eta= 26.6% +6.1% -5.4%

Asymmetric:
./paper_fernandes_2019_broken_powerlaw.py

Best-fit values
  pps= 0.462 +0.0804 -0.0701
  P break= 2.04e+03 +1.16e+03 -1.13e+03
  p1= 0.641 +0.272 -0.146
  p2= -1.29 +0.96 -1.17
  m1= -0.439 +0.0538 -0.0536

  posterior per bin
  eta= 27.4% +7.0% -5.4%

Plots will be generated in the png/ subfolder
png/fernandes2019_rv/occurrence/posterior_x.png 
png/fernandes2019_rv/occurrence/posterior_y.png 
png/fernandes2019_rv_sym/mcmc/triangle.png 

Note that results may vary depending on the random seed, version of the code used, 
and external dependencies. 
Future versions of the code may employ different default parameters.
This script is compatible with version 1.1 of EPOS

If you have trouble running this script or EPOS, please consult the online documentation, 
try running the test and example scripts, or contact the first author

'''
import sys
import EPOS
from EPOS import cgs

Symmetric= 'sym' in sys.argv
Single= 'single' in sys.argv

if Single: suffix='_single'
elif Symmetric: suffix='_sym'
else: suffix=''

''' initialize the EPOS class '''
#Msini = True converts the mass distirbution to an msini distribution
epos= EPOS.epos(name='fernandes2019_rv{}'.format('_sym' if Symmetric else ''), 
	RV = True, MC = False, Msini = True)

''' load the exoplanets and completeness from Mayor+ 2011'''
obs, survey= EPOS.rv.Mayor2011()
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Define a double broken power-law as a planet population '''
if Single:
  epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D_yonly)
elif Symmetric:
	epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D_symmetric)
else:
	epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)

''' Parameter initial guess and fitting ranges
Note: 
	- brokenpowerlaw2D uses 6 parameters, indicated with the is2D keyword
	- 'pps' is a normalization factor for the planet occurrence rate (planets-per-star)
	- parameters a_M and b_M are not fitted
	- dx is the range in walker initial positions for parameters that change sign (+/-)
'''
epos.fitpars.add('pps',		1.0, 	min=1e-3)
if Single:
  epos.fitpars.add('p1',    1.0,  min=0,  max=3,  is2D=True)
else:
  epos.fitpars.add('P break', 1e3,  min=100,max=7e3,is2D=True)
  epos.fitpars.add('p1',    1.0,  min=0,  max=3,  is2D=True)
  if not Symmetric:
    epos.fitpars.add('p2',		-0.5,	min=-3, max=0,	is2D=True)
epos.fitpars.add('M break',	10.0,	fixed=True, 	is2D=True) 
epos.fitpars.add('a_M',		0.0,	fixed=True, 	is2D=True)
epos.fitpars.add('m1',		-0.5,	fixed=False, dx=0.1,	is2D=True)

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[1,1e5],ytrim=[10,1e5],xzoom=[10,1e4],yzoom=[30,6000], 
	Occ=True, UnitTicks=True)

''' Run the simulation once '''
EPOS.run.once(epos)

''' run an MCMC chain on multiple cores or read in a previously saved run'''
EPOS.run.mcmc(epos, nMC=1000, nwalkers=50, nburn=200, threads=20, Saved=True)

''' define bins where occurrence is calculated '''
Pbin= [365.24 * au**1.5 for au in [0.1,100.]]
Mbin= [Mj * cgs.Mjup/cgs.Mearth for Mj in [0.1,13.]] 
epos.set_bins(xbins=[Pbin], ybins=[Mbin])

''' Calculate the occurrence rates '''
EPOS.occurrence.all(epos)

''' Adjust plot parameters'''
epos.plotpars['occrange']= [2e-4,2.]
#epos.xtrim[1]= 4e5

''' plot everything '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos)
EPOS.plot.output.all(epos)
EPOS.plot.mcmc.all(epos)
EPOS.plot.occurrence.all(epos)


AU = 1.49597870691e13                   # Astronomical unit [cgs]
au = 1.49597870691e13                   # Astronomical unit [cgs], as of IAU decree
k = 1.3806503e-16                       # Boltzmann's constant [cgs]
h = 6.62606876e-27                      # Planck's constant [cgs]
c = 2.99792458e10                       # Speed of light [cgs]
micron = 1e-4                           # Micrometer [cgs]
jy = 1e-23                              # Jansky [cgs]
Msun = 1.989e33                         # Solar Mass [cgs]
Rsun = 6.960e10                         # Solar Radius [cgs]
sigma = 5.671e-5                        # Boltzmann Constant [cgs]
G = 6.673e-8                            # Gravitational constant [cgs]
pc = 3.08e18                            # Parsec [cm]
amu = 1.66053886e-24                    # Atomic Mass Unit [g]

min = 60
hour = 3600
day = 86400
year = 3.15569e7                        # google

Rearth = 6.37814e8                      # google
Mearth = 5.97219e27                     # google
rhoEarth= 5.51 							# google
Lsun = 3.839e33                         # google
Psun = 24.47 * day 						# google
Tsun = 5780								# google

Rjup= 69911e5							# google
Mjup= 1.898e30							# google


''' 
Helper function for setting mass-radius distributions
Mass, Radius in earth masses
'''
import numpy as np

def WRF15(mass):
	Mp=np.asarray(mass)
	
	Rgas= (Mp/2.7)**(1/1.3)
	# quick fit to Zeng (see PebbleAccretion/density.py)
	Rrock= Mp**(1./3.7) 

	isrock= Rrock > Rgas
	Rgas[Rgas>12]=12.

	mean=np.where(isrock,Rrock,Rgas)
	#dispersion= np.where(isrock,0.2*Rrock, 1.9/1.3/2.7) # WRF15 estimate
	dispersion=0.2*mean # ~0.5 Rearth at 2.5 Rearth seems reasonable
	
	return mean, dispersion

def CK17(mass):
	''' Chen & Kipping 2017, Forecasting'''
	Mp=np.asarray(mass)
	
	# transitions between rocky, neptune, gas giant
	Mrockmax= 2.0
	Mnepmax= 132 # 0.414 Mjup
	
	#irock= Mp<=Mrockmax
	inep= (Mrockmax<Mp) & (Mp<=Mnepmax)
	igas= Mp>Mnepmax
	
	# Rocky (also sets array)
	Rp= Mp**0.28
	disp= 0.04*Rp
	Rrockmax= Mrockmax**0.28
	
	# Neptunes
	Rp[inep]= Rrockmax * (Mp[inep]/Mrockmax)**0.59
	Rnepmax= Rrockmax * (Mnepmax/Mrockmax)**0.59
	disp[inep]= 0.15*Rp[inep]
	
	# Gas giants
	Rp[igas]= Rnepmax * (Mp[igas]/Mnepmax)**-0.044
	disp[igas]= 0.07*Rp[igas]
	
	return Rp, disp


import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import helpers

''' ellipses don't look very nice'''

def all(name, mass=None, sma=None, inc=None):
	try:
		nsma, nmass, ninc= len(sma), len(mass), len(inc)
	except: 
		raise ValueError('sma, mass, inc have to be iterable\n{} {} {}'.format(
							type(sma), type(mass), type(inc)))
	
	assert nsma == nmass == ninc
	
	for k, (a, m, i) in enumerate(zip (sma, mass, inc)):
		fname= '{}/arch{}'.format(name, k)
		individual(fname, np.asarray(a),np.asarray(m),np.asarray(i))
	
	
def individual(fname, a,m,i, angle=35, ratio= 0.3):
	f, ax = plt.subplots()
	ax.set_title('Architecture')
	ax.set_xlabel('sma')
	ax.set_ylabel('inc[?]')

	ax.set_xlim(-3,1)
	ax.set_ylim(-3,1)
	
	x0, y0= -2, -2
			
	_system(f, ax, a, m, i, x0=x0, y0=y0, angle=angle, ratio=ratio)
	
	helpers.save(plt, 'png/architecture/{}'.format(fname))		

def _system(f, ax, a, m, i, x0=0, y0=0, angle=35, ratio=0.3):
	''' 
	plots an individual system onto a predefined axis 
	NOTE on Ellipse : 
		sma is North-South
		height/width are _diameter_
	'''
	
	amin= np.log10(0.05)-x0
	e= Ellipse((x0, y0), height= 2.*amin, width= 2.*amin*ratio, angle= angle-90, fill=False, ec='b')
	ax.add_artist(e)

	amax= np.log10(1.0)-x0
	e= Ellipse((x0, y0), height= 2.*amax, width= 2.*amax*ratio, angle= angle-90, fill=False, ec='b')
	ax.add_artist(e)


	xx= np.linspace(*ax.get_xlim())
	ax.plot(x0+ (xx-x0)* np.cos(np.pi* (angle)/180.), 
			y0+ (xx-x0)* np.sin(np.pi* (angle)/180.), marker='', ls=':', color='0.5')

	di= i * np.resize([1,-1], i.size) # alternating angles
	sma_x= x0+ (np.log10(a)-x0)* np.cos(np.pi* (angle+0.5*di)/180.)
	sma_y= y0+ (np.log10(a)-x0)* np.sin(np.pi* (angle+0.5*di)/180.)
	
	ax.plot(sma_x, sma_y, ls='', marker='o', color='b')
	
	for _a, _m, _i, _di in zip(a, m, i, di):
		diameter= 2.*(np.log10(_a)-x0)
		e= Ellipse((x0, y0), height=diameter, width=diameter*ratio, 
			angle= angle-90+0.5*_di, 
			fill=False, ec='b',ls=':')
		ax.add_artist(e)
	


'''
Plots the exoplanet survey: observed planets and completeness
'''
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib.colorbar as clrbar

import helpers

def all(epos):
	print '\nPlotting survey...'

	observed(epos, PlotBox=False)
	completeness(epos, PlotBox=False)
	if not epos.RV: 
		completeness(epos, PlotBox=False, Transit=True)
		if hasattr(epos, 'vetting'):
			completeness(epos, PlotBox=False, Transit=True, Vetting=False)
		
	if hasattr(epos, 'vetting'): 
		vetting(epos, PlotBox=False)
		completeness(epos, PlotBox=False, Vetting=False)
	
	if epos.Range and epos.Zoom:
		observed(epos, PlotBox=True)
		completeness(epos, PlotBox=True)
		if not epos.RV: 
			completeness(epos, PlotBox=True, Transit=True)
			if hasattr(epos, 'vetting'):
				completeness(epos, PlotBox=True, Transit=True)
		if hasattr(epos, 'vetting'): 
			vetting(epos, PlotBox=True)
			completeness(epos, PlotBox=True, Vetting=False)

	if hasattr(epos,'obs_score'):
		observed(epos, PlotBox=False, PlotScore=True)

def observed(epos, PlotBox=True, PlotScore=False):
	assert epos.Observation

	if PlotScore:
		f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
		f.subplots_adjust(wspace=0)
	else:
		f, ax = plt.subplots()
	ax.set_title('Observed Population')
	
	helpers.set_axes(ax, epos, Trim=epos.Range)
	
	if PlotScore:
		''' color scale? '''
		cmap='plasma' # viridis, plasma, inferno, magma, spring, cool
		vmin, vmax= 0, 1
		#ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
		clrs, norm= helpers.color_array(epos.obs_score,
			vmin=vmin,vmax=vmax, cmap=cmap)
		ax.scatter(epos.obs_xvar, epos.obs_yvar, color=clrs, s=3)

		# colorbar?
		cb1 = clrbar.ColorbarBase(axb, cmap=cmap, norm=norm,
	                                orientation='vertical') # horizontal
		axb.tick_params(axis='y', direction='out')

		fname='.score'
	else:
		ax.plot(epos.obs_xvar, epos.obs_yvar, ls='', marker='.', mew=0, ms=5.0, color='k')
		fname=''
		# add multis? 
	
	if PlotBox:
		fname='.box'
		assert epos.Range
		ax.add_patch(patches.Rectangle( (epos.xtrim[0],epos.ytrim[0]), 
			epos.xtrim[1]-epos.xtrim[0], epos.ytrim[1]-epos.ytrim[0],fill=False, zorder=1, ls='--') )
		if epos.Zoom:
			ax.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
				epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )

	helpers.save(plt, epos.plotdir+'survey/planets'+fname)
	
def completeness(epos, PlotBox=False, Transit=False, Vetting=True):
	assert epos.DetectionEfficiency

	f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
	f.subplots_adjust(wspace=0)
	#f.set_size_inches(7, 5)

	ax.set_title('Detection Efficiency' if Transit else 'Survey Completeness')
	helpers.set_axes(ax, epos, Trim=False, Eff=True)
	
	if Transit:
		toplot= epos.eff_2D
		if Vetting and hasattr(epos, 'vetting'): toplot*= epos.vetting
	else:
		toplot= epos.completeness if Vetting else epos.completeness_novet
	
	with np.errstate(divide='ignore'): log_completeness=  np.log10(toplot)
	
	''' color map and ticks'''
	cmap = 'YlOrBr' if Transit else 'PuRd'
	#cmap = 'rainbow' if Transit else 'PuRd'

	vmin, vmax= -4, 0
	if Transit: vmin=-3
	ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
	levels=np.linspace(vmin, vmax, 256)

	cs= ax.contourf(epos.eff_xvar, epos.eff_yvar,log_completeness.T, cmap=cmap, levels=levels, vmin=vmin, vmax=vmax)

	cbar= f.colorbar(cs, cax=axb, ticks=ticks)
	axb.set_yticklabels(100*10.**ticks)
	axb.tick_params(axis='y', direction='out')
	axb.set_title('%')
	
	''' Plot the zoom box or a few black contours'''	
	if PlotBox:
		fname='.box'
		assert epos.Range
		ax.add_patch(patches.Rectangle( (epos.xtrim[0],epos.ytrim[0]), 
			epos.xtrim[1]-epos.xtrim[0], epos.ytrim[1]-epos.ytrim[0],fill=False, zorder=1, ls='--') )
		if epos.Zoom:
			ax.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
				epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )
	else:
		if hasattr(epos, 'xtrim'):
			ax.set_xlim(*epos.xtrim)
			ax.set_ylim(*epos.ytrim)
		
		if epos.RV:
			levels= [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9]
			labels= ['10','20','30','40','50','60','70','80','90']
			cs= ax.contour(epos.eff_xvar, epos.eff_yvar, toplot.T,\
					levels= levels, colors = 'k', linewidths=2.)
		else:
			levels= [1e-3, 0.01, 0.1, 0.5, 0.9] if Transit else 10.**ticks
			cs= ax.contour(epos.eff_xvar, epos.eff_yvar, toplot.T, 
				colors='k', levels=levels)
			fmt_percent= lambda x: '{:g} %'.format(100.*x)
			plt.clabel(cs, cs.levels, inline=True, fmt=fmt_percent)

		fname=''

	if not Vetting: fname+= '.novet'
	helpers.save(plt, epos.plotdir+'survey/'+('efficiency' if Transit else 'completeness')+ \
				fname)

def vetting(epos, PlotBox=False):
	assert hasattr(epos, 'vetting')

	f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
	f.subplots_adjust(wspace=0)
	#f.set_size_inches(7, 5)

	ax.set_title('Vetting Efficiency')
	helpers.set_axes(ax, epos, Trim=False, Eff=True)
	

	''' color map and ticks'''
	cmap = 'plasma'
	levels=np.linspace(0,1, 256)
	ticks= np.array([0.0,0.25,0.5,0.75,1.0])
	ticks_percent=['0','25','50','75','100']

	cs= ax.contourf(epos.eff_xvar, epos.eff_yvar,epos.vetting.T, cmap=cmap, levels=levels)

	cbar= f.colorbar(cs, cax=axb, ticks=ticks)
	axb.set_yticklabels(ticks_percent)
	axb.tick_params(axis='y', direction='out')
	axb.set_title('%')
	
	''' Plot the zoom box or a few black contours'''	
	if PlotBox:
		fname='.box'
		assert epos.Range
		ax.add_patch(patches.Rectangle( (epos.xtrim[0],epos.ytrim[0]), 
			epos.xtrim[1]-epos.xtrim[0], epos.ytrim[1]-epos.ytrim[0],fill=False, zorder=1, ls='--') )
		if epos.Zoom:
			ax.add_patch(patches.Rectangle( (epos.xzoom[0],epos.yzoom[0]), 
				epos.xzoom[1]-epos.xzoom[0], epos.yzoom[1]-epos.yzoom[0],fill=False, zorder=1) )
	else:
	
		cs= ax.contour(epos.eff_xvar, epos.eff_yvar, epos.vetting.T, 
			colors='k', levels=ticks)
		fmt_percent= lambda x: '{:g} %'.format(100.*x)
		plt.clabel(cs, cs.levels, inline=True, fmt=fmt_percent)

		fname=''
		if hasattr(epos, 'xtrim'):		
			ax.set_xlim(*epos.xtrim)
			ax.set_ylim(*epos.ytrim)

	helpers.save(plt, epos.plotdir+'survey/vetting'+ fname)



import numpy as np
import scipy.stats # norm
import EPOS.fitfunctions
from scipy.interpolate import interp2d, RectBivariateSpline

def periodradius(epos, Init=False, fpara=None, fdet=None, 
			xbin=None, ybin=None, xgrid=None, ygrid=None, Convert=False):
	''' 
	Return the period-radius distribution
	in referse to the model parameter (Mass, Msini or radius)
	out reverse to the observed parameter (Msini or Radius)
	default is the input grid. 
	For MC=False, the distribution can also be Converted onto the ouput grid (M->Msini or M->R)
	'''
	if fpara is None:
		pps= epos.pdfpars.getpps(Init=Init)
		fpar2d= epos.pdfpars.get2d(Init=Init)
	else:
		pps= epos.pdfpars.getpps_fromlist(fpara)
		fpar2d= epos.pdfpars.get2d_fromlist(fpara)
		#print fpara
	
	pdf= epos.func(epos.X_in, epos.Y_in, *fpar2d)
	pdf_X, pdf_Y= np.sum(pdf, axis=1), np.sum(pdf, axis=0)
	sum_pdf= np.sum(pdf)
	sum_pdf_X= np.sum(pdf_X)
	sum_pdf_Y= np.sum(pdf_Y)

	''' Convert M->Msini or M->R'''
	if Convert:
		if epos.RV:
			pdf= _convert_pdf_M_to_Msini(epos.MC_xvar, epos.in_yvar, epos.MC_yvar, pdf)
			pdf_X, pdf_Y= np.sum(pdf, axis=1), np.sum(pdf, axis=0)
		elif epos.MR:
			pass

	# convolve with detection efficiency?
	if fdet is not None:
		det_pdf= pdf*fdet
		det_pdf_X, det_pdf_Y= np.sum(det_pdf, axis=1), np.sum(det_pdf, axis=0)
	
	# calculate pdf on different grid?
	if xbin is not None:
		xgrid= np.logspace(np.log10(xbin[0]),np.log10(xbin[-1]),5)
	if ybin is not None:
		ygrid= np.logspace(np.log10(ybin[0]),np.log10(ybin[-1]),5)
	
	# calculate pdf on provided grid
	if (xgrid is not None) or (ygrid is not None):
		#assert Convert==False, 'Not implemented'
		if xgrid is None:
			xgrid= epos.MC_xvar
		if ygrid is None:
			#ygrid= epos.MC_yvar
			ygrid= epos.in_yvar
	
		X,Y=np.meshgrid(xgrid, ygrid,indexing='ij')
		pdf= epos.func(X,Y, *fpar2d)
		
		# normalized per unit dlnxdlny
		pdf= pps* pdf/sum_pdf* epos.scale

		if Convert:
			assert np.all(ygrid==epos.MC_yvar) or np.all(ygrid==epos.in_yvar), 'cannot do custom y grid'
			pdf= _convert_pdf_M_to_Msini(xgrid, ygrid, epos.MC_yvar, pdf)
			#pdf_X, pdf_Y= np.sum(pdf, axis=1), np.sum(pdf, axis=0)

		if fdet is not None:
			#assert False, 'Where is this used?'
			#func_fdet= interp2d(xgrid, ygrid, fdet.T, kind='cubic')
			func_fdet= RectBivariateSpline(epos.MC_xvar, epos.MC_yvar, fdet) # was in_yvar
			_fdet= func_fdet(xgrid, ygrid)
			#print fdet.shape, _fdet.shape
			#print xgrid
			#print ygrid
			#print _fdet
			pdf *= _fdet

		dlnx= np.log(xgrid[-1]/xgrid[0])
		dlny= np.log(ygrid[-1]/ygrid[0])

		pdf_X= np.average(pdf, axis=1) * dlny
		pdf_Y= np.average(pdf, axis=0) * dlnx

	else:
		# calculate pdf on default grid
		# normalized in units dlnx, dlny, and dlnxdlny
		if fdet is None:
			pdf_X= pps* pdf_X/sum_pdf_X * epos.scale_x
			pdf_Y= pps* pdf_Y/sum_pdf_Y * epos.scale_in_y
			pdf= pps* pdf/sum_pdf* epos.scale
		else:
			pdf_X= pps* det_pdf_X/sum_pdf_X * epos.scale_x
			pdf_Y= pps* det_pdf_Y/sum_pdf_Y * epos.scale_in_y
			pdf= pps* det_pdf/sum_pdf* epos.scale
	
	return pps, pdf, pdf_X, pdf_Y

def periodratio(epos, Pgrid=None, fpara=None, Init=False):
	
	if fpara is None:
		if epos.spacing == 'powerlaw':
			dPbreak= epos.fitpars.get('dP break')
			dP1= epos.fitpars.get('dP 1')
			dP2= epos.fitpars.get('dP 2')
			pdf= EPOS.fitfunctions.brokenpowerlaw1D(Pgrid, dPbreak, dP1, dP2)
		elif epos.spacing=='dimensionless':
			logD=  epos.fitpars.get('log D')
			sigma= epos.fitpars.get('sigma')

			with np.errstate(divide='ignore'): 
				Dgrid= np.log10(2.*(Pgrid**(2./3.)-1.)/(Pgrid**(2./3.)+1.))
			Dgrid[0]= -2
			#print Dgrid
			pdf= scipy.stats.norm(logD,sigma).pdf(Dgrid)
			
		cdf= np.cumsum(pdf)
		cdf/=cdf[-1]
	else:
		# copy-pasta from run.py
		raise ValueError('test needed')
	
	return pdf, cdf

''' Helper function '''
def _int_fmsini(x):
	''' Integral of x/sqrt(1.-x**2.) to avoid infinite value at x=1'''
   	return -np.sqrt(1.-x**2.)

def _convert_pdf_M_to_Msini(xgrid, in_ygrid, out_ygrid, pdf):
	'''Generate the shape of the Msini distribution'''
	fint1d= _int_fmsini(in_ygrid[1:]/in_ygrid[-1])-_int_fmsini(in_ygrid[:-1]/in_ygrid[-1])
	fint2d= np.tile(fint1d, (xgrid.size,1))
	#print 'fint 1d {}'.format(fint1d.shape)
	#print 'fint 2d {}'.format(fint2d.shape)
	#print 'pdf {}'.format(pdf.shape)

	''' Convolve the the pdf with the msini distribution through shift and add'''	
	pdf_ext= np.zeros_like(pdf) # extended grid
	for i in range(in_ygrid.size-1):
		fobsi= pdf[:,i][:, np.newaxis]*fint2d[:,-(i+1):] # matrix multiplication.  
		pdf_ext[:, :(i+1)]+= fobsi

	''' Truncate the pdf to the mass range of Msini'''
	pdf_out= pdf_ext[:,:out_ygrid.size] # this in on out grid
	#print 'pdf before {}, after {}'.format(pdf.shape, pdf_ext[:,:out_ygrid.size].shape)
	return pdf_out 